@@ -1,4 +1,4 @@
-#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2023, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -28,8 +28,8 @@ import os
 import sys
 
 from nuitka.build.DataComposerInterface import runDataComposer
-from nuitka.build.SconsUtils import getSconsReportValue
-from nuitka.constants.Serialization import ConstantAccessor
+from nuitka.build.SconsUtils import getSconsReportValue, readSconsReport
+from nuitka.code_generation.ConstantCodes import addDistributionMetadataValue
 from nuitka.freezer.IncludedDataFiles import (
     addIncludedDataFilesFromFileOptions,
     addIncludedDataFilesFromPackageOptions,
@@ -69,10 +69,12 @@ from nuitka.PythonVersions import (
     python_version,
     python_version_str,
 )
+from nuitka.Serialization import ConstantAccessor
 from nuitka.Tracing import general, inclusion_logger
 from nuitka.tree import SyntaxErrors
 from nuitka.tree.ReformulationMultidist import createMultidistMainSourceCode
 from nuitka.utils import InstanceCounters
+from nuitka.utils.Distributions import getDistribution
 from nuitka.utils.Execution import (
     callProcess,
     withEnvironmentVarOverridden,
@@ -161,6 +163,17 @@ def _createMainModule():
 
     OutputDirectories.setMainModule(main_module)
 
+    for distribution_name in Options.getShallIncludeDistributionMetadata():
+        distribution = getDistribution(distribution_name)
+
+        if distribution is None:
+            general.sysexit(
+                "Error, could not find distribution '%s' for which metadata was asked to be included."
+                % distribution_name
+            )
+
+        addDistributionMetadataValue(distribution_name, distribution)
+
     # First remove old object files and old generated files, old binary or
     # module, and standalone mode program directory if any, they can only do
     # harm.
@@ -270,6 +283,8 @@ def dumpTreeXML():
             for module in ModuleRegistry.getDoneModules():
                 dumpTreeXMLToFile(tree=module.asXml(), output_file=output_file)
 
+        general.info("XML dump of node state written to file '%s'." % filename)
+
 
 def pickSourceFilenames(source_dir, modules):
     """Pick the names for the C files of each module.
@@ -1023,6 +1038,8 @@ def _main():
     if Options.isRemoveBuildDir():
         general.info("Removing build directory '%s'." % source_dir)
 
+        # Make sure the scons report is cached before deleting it.
+        readSconsReport(source_dir)
         removeDirectory(path=source_dir, ignore_errors=False)
         assert not os.path.exists(source_dir)
     else:
