@@ -1,4 +1,4 @@
-#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2023, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -161,6 +161,10 @@ else:
 # Just to make this not Windows-specific.
 WindowsError = OSError  # pylint: disable=I0021,redefined-builtin
 
+# Make it available for Python2 as well
+PermissionError = (  # pylint: disable=redefined-builtin
+    PermissionError if str is not bytes else OSError
+)
 
 if not hasattr(pkgutil, "ModuleInfo"):
     # Python3.5 or lower do not return namedtuple, but it's nicer to read code with it.
@@ -176,6 +180,11 @@ else:
     iter_modules = pkgutil.iter_modules
 
 
+try:
+    ExceptionGroup = ExceptionGroup
+except NameError:
+    ExceptionGroup = None
+
 # For PyLint to be happy.
 assert long
 assert unicode
