@@ -1,4 +1,4 @@
-#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2023, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -80,7 +80,12 @@ from .BuiltinCodes import (
     generateBuiltinXrange3Code,
 )
 from .CallCodes import generateCallCode, getCallsCode
-from .ClassCodes import generateBuiltinSuperCode, generateSelectMetaclassCode
+from .ClassCodes import (
+    generateBuiltinSuper1Code,
+    generateBuiltinSuperCode,
+    generateSelectMetaclassCode,
+    generateTypeOperationPrepareCode,
+)
 from .CodeHelpers import addExpressionDispatchDict, setStatementDispatchDict
 from .ComparisonCodes import (
     generateBuiltinIsinstanceCode,
@@ -89,6 +94,7 @@ from .ComparisonCodes import (
     generateMatchTypeCheckMappingCode,
     generateMatchTypeCheckSequenceCode,
     generateRichComparisonExpressionCode,
+    generateSubtypeCheckCode,
     generateTypeCheckCode,
 )
 from .ConditionalCodes import (
@@ -114,6 +120,9 @@ from .DictCodes import (
     generateDictionaryCreationCode,
     generateDictOperationClearCode,
     generateDictOperationCopyCode,
+    generateDictOperationFromkeys2Code,
+    generateDictOperationFromkeys3Code,
+    generateDictOperationFromkeysRefCode,
     generateDictOperationGet2Code,
     generateDictOperationGet3Code,
     generateDictOperationInCode,
@@ -218,8 +227,22 @@ from .ListCodes import (
     generateBuiltinListCode,
     generateListCreationCode,
     generateListOperationAppendCode,
+    generateListOperationAppendCode2,
+    generateListOperationClearCode,
+    generateListOperationCopyCode,
+    generateListOperationCountCode,
     generateListOperationExtendCode,
-    generateListOperationPopCode,
+    generateListOperationIndex2Code,
+    generateListOperationIndex3Code,
+    generateListOperationIndex4Code,
+    generateListOperationInsertCode,
+    generateListOperationPop1Code,
+    generateListOperationPop2Code,
+    generateListOperationRemoveCode,
+    generateListOperationReverseCode,
+    generateListOperationSort1Code,
+    generateListOperationSort2Code,
+    generateListOperationSort3Code,
 )
 from .LocalsDictCodes import (
     generateLocalsDictDelCode,
@@ -229,6 +252,7 @@ from .LocalsDictCodes import (
     generateLocalsDictVariableRefOrFallbackCode,
     generateReleaseLocalsDictCode,
     generateSetLocalsDictCode,
+    generateSetLocalsMappingCode,
 )
 from .LoopCodes import (
     generateLoopBreakCode,
@@ -248,13 +272,25 @@ from .OperationCodes import (
     generateOperationUnaryCode,
 )
 from .PackageResourceCodes import (
+    generateImportlibMetadataBackportEntryPointsCallCode,
+    generateImportlibMetadataBackportEntryPointsValueCode,
+    generateImportlibMetadataBackportEntryPointValueCode,
+    generateImportlibMetadataBackportSelectableGroupsValueCode,
     generateImportlibMetadataBackportVersionCallCode,
     generateImportlibMetadataDistributionCallCode,
     generateImportlibMetadataDistributionValueCode,
+    generateImportlibMetadataEntryPointsSince310CallCode,
+    generateImportlibMetadataEntryPointsValueCode,
+    generateImportlibMetadataEntryPointValueCode,
+    generateImportlibMetadataSelectableGroupsValueCode,
     generateImportlibMetadataVersionCallCode,
     generateImportlibResourcesReadBinaryCallCode,
     generateImportlibResourcesReadTextCallCode,
+    generateOsListdirCallCode,
+    generateOsPathAbspathCallCode,
+    generateOsPathBasenameCallCode,
     generateOsPathExistsCallCode,
+    generateOsPathIsabsCallCode,
     generateOsPathIsdirCallCode,
     generateOsPathIsfileCallCode,
     generateOsUnameCallCode,
@@ -346,7 +382,7 @@ def generateFunctionBodyCode(function_body, context):
         function_context = Contexts.PythonGeneratorObjectContext(
             parent=context, function=function_body
         )
-    elif function_body.isExpressionClassBody():
+    elif function_body.isExpressionClassBodyBase():
         function_context = Contexts.PythonFunctionDirectContext(
             parent=context, function=function_body
         )
@@ -419,7 +455,7 @@ def generateFunctionBodyCode(function_body, context):
             closure_variables=function_body.getClosureVariables(),
         )
 
-    elif function_body.isExpressionClassBody():
+    elif function_body.isExpressionClassBodyBase():
         function_code = getFunctionCode(
             context=function_context,
             function_identifier=function_identifier,
@@ -488,13 +524,14 @@ def _generateModuleCode(module, data_filename):
     function_body_codes = []
 
     for function_body in module.getUsedFunctions():
-        # Constant function returners get no code.
-        (
-            is_constant_returning,
-            _constant_return_value,
-        ) = function_body.getConstantReturnValue()
-        if is_constant_returning:
-            continue
+        if function_body.needsCreation():
+            # Constant function returners get no code.
+            (
+                is_constant_returning,
+                _constant_return_value,
+            ) = function_body.getConstantReturnValue()
+            if is_constant_returning:
+                continue
 
         function_code, function_decl = generateFunctionBodyCode(
             function_body=function_body, context=context
@@ -554,6 +591,9 @@ def generateHelpersCode():
     )
 
 
+# TODO: Some of these have names that are way too long, and this should be more automatic in
+# standard cases, e.g. through generation.
+# pylint: disable=line-too-long
 addExpressionDispatchDict(
     {
         "EXPRESSION_ATTRIBUTE_CHECK": generateAttributeCheckCode,
@@ -609,12 +649,15 @@ addExpressionDispatchDict(
         "EXPRESSION_BUILTIN_LOCALS_COPY": generateBuiltinLocalsCode,
         "EXPRESSION_BUILTIN_LOCALS_UPDATED": generateBuiltinLocalsCode,
         "EXPRESSION_BUILTIN_LOCALS_REF": generateBuiltinLocalsRefCode,
+        "EXPRESSION_LOCALS_DICT_REF": generateBuiltinLocalsRefCode,
         "EXPRESSION_BUILTIN_GLOBALS": generateBuiltinGlobalsCode,
         "EXPRESSION_BUILTIN_SUPER0": generateBuiltinSuperCode,
         "EXPRESSION_BUILTIN_SUPER2": generateBuiltinSuperCode,
+        "EXPRESSION_BUILTIN_SUPER1": generateBuiltinSuper1Code,
         "EXPRESSION_BUILTIN_ISINSTANCE": generateBuiltinIsinstanceCode,
         "EXPRESSION_BUILTIN_ISSUBCLASS": generateBuiltinIssubclassCode,
         "EXPRESSION_TYPE_CHECK": generateTypeCheckCode,
+        "EXPRESSION_SUBTYPE_CHECK": generateSubtypeCheckCode,
         "EXPRESSION_MATCH_TYPE_CHECK_SEQUENCE": generateMatchTypeCheckSequenceCode,
         "EXPRESSION_MATCH_TYPE_CHECK_MAPPING": generateMatchTypeCheckMappingCode,
         "EXPRESSION_BUILTIN_DIR1": generateBuiltinDir1Code,
@@ -622,7 +665,8 @@ addExpressionDispatchDict(
         "EXPRESSION_BUILTIN_HASATTR": generateBuiltinHasattrCode,
         "EXPRESSION_BUILTIN_GETATTR": generateBuiltinGetattrCode,
         "EXPRESSION_BUILTIN_SETATTR": generateBuiltinSetattrCode,
-        "EXPRESSION_BUILTIN_OPEN": generateBuiltinOpenCode,
+        "EXPRESSION_BUILTIN_OPEN_P2": generateBuiltinOpenCode,
+        "EXPRESSION_BUILTIN_OPEN_P3": generateBuiltinOpenCode,
         "EXPRESSION_BUILTIN_STATICMETHOD": generateBuiltinStaticmethodCode,
         "EXPRESSION_BUILTIN_CLASSMETHOD": generateBuiltinClassmethodCode,
         "EXPRESSION_BUILTIN_RANGE1": generateBuiltinRange1Code,
@@ -672,7 +716,12 @@ addExpressionDispatchDict(
         "EXPRESSION_CONSTANT_SLICE_REF": generateConstantReferenceCode,
         "EXPRESSION_CONSTANT_XRANGE_REF": generateConstantReferenceCode,
         "EXPRESSION_CONSTANT_TYPE_REF": generateConstantReferenceCode,
-        "EXPRESSION_CONSTANT_TYPE_SUBSCRIPTABLE_REF": generateConstantReferenceCode,
+        "EXPRESSION_CONSTANT_TYPE_DICT_REF": generateConstantReferenceCode,
+        "EXPRESSION_CONSTANT_TYPE_SET_REF": generateConstantReferenceCode,
+        "EXPRESSION_CONSTANT_TYPE_FROZENSET_REF": generateConstantReferenceCode,
+        "EXPRESSION_CONSTANT_TYPE_LIST_REF": generateConstantReferenceCode,
+        "EXPRESSION_CONSTANT_TYPE_TUPLE_REF": generateConstantReferenceCode,
+        "EXPRESSION_CONSTANT_TYPE_TYPE_REF": generateConstantReferenceCode,
         "EXPRESSION_CONSTANT_BYTEARRAY_REF": generateConstantReferenceCode,
         "EXPRESSION_CONSTANT_GENERIC_ALIAS": generateConstantGenericAliasCode,
         "EXPRESSION_CONSTANT_UNION_TYPE": generateConstantReferenceCode,
@@ -717,7 +766,11 @@ addExpressionDispatchDict(
         "EXPRESSION_DICT_OPERATION_POPITEM": generateDictOperationPopitemCode,
         "EXPRESSION_DICT_OPERATION_UPDATE2": generateDictOperationUpdate2Code,
         "EXPRESSION_DICT_OPERATION_UPDATE3": generateDictOperationUpdate3Code,
+        "EXPRESSION_DICT_OPERATION_UPDATE_PAIRS": generateDictOperationUpdate3Code,
+        "EXPRESSION_DICT_OPERATION_FROMKEYS2": generateDictOperationFromkeys2Code,
+        "EXPRESSION_DICT_OPERATION_FROMKEYS3": generateDictOperationFromkeys3Code,
         "EXPRESSION_FUNCTION_CREATION": generateFunctionCreationCode,
+        "EXPRESSION_FUNCTION_CREATION_OLD": generateFunctionCreationCode,
         "EXPRESSION_FUNCTION_CALL": generateFunctionCallCode,
         "EXPRESSION_FUNCTION_ERROR_STR": generateFunctionErrorStrCode,
         "EXPRESSION_IMPORT_MODULE_FIXED": generateImportModuleFixedCode,
@@ -727,9 +780,23 @@ addExpressionDispatchDict(
         "EXPRESSION_IMPORTLIB_IMPORT_MODULE_REF": generateImportModuleNameHardCode,
         "EXPRESSION_IMPORTLIB_IMPORT_MODULE_CALL": generateImportlibImportCallCode,
         "EXPRESSION_IMPORT_NAME": generateImportNameCode,
+        "EXPRESSION_LIST_OPERATION_APPEND": generateListOperationAppendCode2,
         "EXPRESSION_LIST_OPERATION_EXTEND": generateListOperationExtendCode,
         "EXPRESSION_LIST_OPERATION_EXTEND_FOR_UNPACK": generateListOperationExtendCode,
-        "EXPRESSION_LIST_OPERATION_POP": generateListOperationPopCode,
+        "EXPRESSION_LIST_OPERATION_CLEAR": generateListOperationClearCode,
+        "EXPRESSION_LIST_OPERATION_COPY": generateListOperationCopyCode,
+        "EXPRESSION_LIST_OPERATION_COUNT": generateListOperationCountCode,
+        "EXPRESSION_LIST_OPERATION_INSERT": generateListOperationInsertCode,
+        "EXPRESSION_LIST_OPERATION_INDEX2": generateListOperationIndex2Code,
+        "EXPRESSION_LIST_OPERATION_INDEX3": generateListOperationIndex3Code,
+        "EXPRESSION_LIST_OPERATION_INDEX4": generateListOperationIndex4Code,
+        "EXPRESSION_LIST_OPERATION_POP1": generateListOperationPop1Code,
+        "EXPRESSION_LIST_OPERATION_POP2": generateListOperationPop2Code,
+        "EXPRESSION_LIST_OPERATION_REMOVE": generateListOperationRemoveCode,
+        "EXPRESSION_LIST_OPERATION_REVERSE": generateListOperationReverseCode,
+        "EXPRESSION_LIST_OPERATION_SORT1": generateListOperationSort1Code,
+        "EXPRESSION_LIST_OPERATION_SORT2": generateListOperationSort2Code,
+        "EXPRESSION_LIST_OPERATION_SORT3": generateListOperationSort3Code,
         "EXPRESSION_MODULE_ATTRIBUTE_FILE_REF": generateModuleAttributeFileCode,
         "EXPRESSION_MODULE_ATTRIBUTE_NAME_REF": generateModuleAttributeCode,
         "EXPRESSION_MODULE_ATTRIBUTE_PACKAGE_REF": generateModuleAttributeCode,
@@ -781,7 +848,8 @@ addExpressionDispatchDict(
         "EXPRESSION_OUTLINE_BODY": generateFunctionOutlineCode,
         "EXPRESSION_OUTLINE_FUNCTION": generateFunctionOutlineCode,
         # TODO: Rename to make more clear it is an outline
-        "EXPRESSION_CLASS_BODY": generateFunctionOutlineCode,
+        "EXPRESSION_CLASS_BODY_P2": generateFunctionOutlineCode,
+        "EXPRESSION_CLASS_BODY_P3": generateFunctionOutlineCode,
         "EXPRESSION_SUBSCRIPT_LOOKUP": generateSubscriptLookupCode,
         "EXPRESSION_SUBSCRIPT_LOOKUP_FOR_UNPACK": generateSubscriptLookupCode,
         "EXPRESSION_SUBSCRIPT_CHECK": generateSubscriptCheckCode,
@@ -809,7 +877,7 @@ addExpressionDispatchDict(
         "EXPRESSION_LOCALS_VARIABLE_REF": generateLocalsDictVariableRefCode,
         "EXPRESSION_RAISE_EXCEPTION": generateRaiseExpressionCode,
         "EXPRESSION_NUITKA_LOADER_CREATION": generateNuitkaLoaderCreationCode,
-        "EXPRESSION_PKGLIB_GET_DATA_REF": generateImportModuleNameHardCode,
+        "EXPRESSION_PKGUTIL_GET_DATA_REF": generateImportModuleNameHardCode,
         "EXPRESSION_PKG_RESOURCES_REQUIRE_REF": generateImportModuleNameHardCode,
         "EXPRESSION_PKG_RESOURCES_GET_DISTRIBUTION_REF": generateImportModuleNameHardCode,
         "EXPRESSION_PKG_RESOURCES_ITER_ENTRY_POINTS_REF": generateImportModuleNameHardCode,
@@ -823,16 +891,30 @@ addExpressionDispatchDict(
         "EXPRESSION_IMPORTLIB_METADATA_BACKPORT_VERSION_REF": generateImportModuleNameHardCode,
         "EXPRESSION_IMPORTLIB_METADATA_DISTRIBUTION_REF": generateImportModuleNameHardCode,
         "EXPRESSION_IMPORTLIB_METADATA_BACKPORT_DISTRIBUTION_REF": generateImportModuleNameHardCode,
+        "EXPRESSION_IMPORTLIB_METADATA_ENTRY_POINTS_REF": generateImportModuleNameHardCode,
+        "EXPRESSION_IMPORTLIB_METADATA_BACKPORT_ENTRY_POINTS_REF": generateImportModuleNameHardCode,
         "EXPRESSION_IMPORTLIB_METADATA_METADATA_REF": generateImportModuleNameHardCode,
         "EXPRESSION_IMPORTLIB_METADATA_BACKPORT_METADATA_REF": generateImportModuleNameHardCode,
         "EXPRESSION_IMPORTLIB_METADATA_DISTRIBUTION_VALUE_REF": generateImportlibMetadataDistributionValueCode,
+        "EXPRESSION_IMPORTLIB_METADATA_ENTRY_POINT_VALUE_REF": generateImportlibMetadataEntryPointValueCode,
+        "EXPRESSION_IMPORTLIB_METADATA_BACKPORT_ENTRY_POINT_VALUE_REF": generateImportlibMetadataBackportEntryPointValueCode,
+        "EXPRESSION_IMPORTLIB_METADATA_SELECTABLE_GROUPS_VALUE_REF": generateImportlibMetadataSelectableGroupsValueCode,
+        "EXPRESSION_IMPORTLIB_METADATA_BACKPORT_SELECTABLE_GROUPS_VALUE_REF": generateImportlibMetadataBackportSelectableGroupsValueCode,
+        "EXPRESSION_IMPORTLIB_METADATA_ENTRY_POINTS_VALUE_REF": generateImportlibMetadataEntryPointsValueCode,
+        "EXPRESSION_IMPORTLIB_METADATA_BACKPORT_ENTRY_POINTS_VALUE_REF": generateImportlibMetadataBackportEntryPointsValueCode,
+        "EXPRESSION_IMPORTLIB_METADATA_ENTRY_POINTS_BEFORE310_CALL": generateImportlibMetadataEntryPointsSince310CallCode,
+        "EXPRESSION_IMPORTLIB_METADATA_ENTRY_POINTS_SINCE310_CALL": generateImportlibMetadataEntryPointsSince310CallCode,
+        "EXPRESSION_IMPORTLIB_METADATA_BACKPORT_ENTRY_POINTS_CALL": generateImportlibMetadataBackportEntryPointsCallCode,
         "EXPRESSION_OS_UNAME_REF": generateImportModuleNameHardCode,
+        "EXPRESSION_OS_LISTDIR_REF": generateImportModuleNameHardCode,
         "EXPRESSION_OS_PATH_EXISTS_REF": generateImportModuleNameHardCode,
         "EXPRESSION_OS_PATH_ISFILE_REF": generateImportModuleNameHardCode,
         "EXPRESSION_OS_PATH_ISDIR_REF": generateImportModuleNameHardCode,
+        "EXPRESSION_OS_PATH_BASENAME_REF": generateImportModuleNameHardCode,
         "EXPRESSION_CTYPES_CDLL_REF": generateImportModuleNameHardCode,
-        "EXPRESSION_CTYPES_CDLL_CALL": generateCtypesCdllCallCode,
-        "EXPRESSION_PKGLIB_GET_DATA_CALL": generatePkglibGetDataCallCode,
+        "EXPRESSION_CTYPES_CDLL_SINCE38_CALL": generateCtypesCdllCallCode,
+        "EXPRESSION_CTYPES_CDLL_BEFORE38_CALL": generateCtypesCdllCallCode,
+        "EXPRESSION_PKGUTIL_GET_DATA_CALL": generatePkglibGetDataCallCode,
         "EXPRESSION_PKG_RESOURCES_REQUIRE_CALL": generatePkgResourcesRequireCallCode,
         "EXPRESSION_PKG_RESOURCES_GET_DISTRIBUTION_CALL": generatePkgResourcesGetDistributionCallCode,
         "EXPRESSION_PKG_RESOURCES_ITER_ENTRY_POINTS_CALL": generatePkgResourcesIterEntryPointsCallCode,
@@ -848,8 +930,15 @@ addExpressionDispatchDict(
         "EXPRESSION_OS_PATH_EXISTS_CALL": generateOsPathExistsCallCode,
         "EXPRESSION_OS_PATH_ISFILE_CALL": generateOsPathIsfileCallCode,
         "EXPRESSION_OS_PATH_ISDIR_CALL": generateOsPathIsdirCallCode,
+        "EXPRESSION_OS_PATH_BASENAME_CALL": generateOsPathBasenameCallCode,
+        "EXPRESSION_OS_PATH_ABSPATH_CALL": generateOsPathAbspathCallCode,
+        "EXPRESSION_OS_PATH_ISABS_CALL": generateOsPathIsabsCallCode,
+        "EXPRESSION_OS_LISTDIR_CALL": generateOsListdirCallCode,
         "EXPRESSION_MATCH_ARGS": generateMatchArgsCode,
         "EXPRESSION_STR_OPERATION_FORMAT": generateStrFormatMethodCode,
+        # TODO: Should have all of these generically or not. This one is required for now.
+        "EXPRESSION_DICT_OPERATION_FROMKEYS_REF": generateDictOperationFromkeysRefCode,
+        "EXPRESSION_TYPE_OPERATION_PREPARE": generateTypeOperationPrepareCode,
     }
 )
 
@@ -878,7 +967,9 @@ setStatementDispatchDict(
     {
         "STATEMENT_ASSIGNMENT_VARIABLE_GENERIC": generateAssignmentVariableCode,
         "STATEMENT_ASSIGNMENT_VARIABLE_CONSTANT_MUTABLE": generateAssignmentVariableCode,
+        "STATEMENT_ASSIGNMENT_VARIABLE_CONSTANT_MUTABLE_TRUSTED": generateAssignmentVariableCode,
         "STATEMENT_ASSIGNMENT_VARIABLE_CONSTANT_IMMUTABLE": generateAssignmentVariableCode,
+        "STATEMENT_ASSIGNMENT_VARIABLE_CONSTANT_IMMUTABLE_TRUSTED": generateAssignmentVariableCode,
         "STATEMENT_ASSIGNMENT_VARIABLE_ITERATOR": generateAssignmentVariableCode,
         "STATEMENT_ASSIGNMENT_VARIABLE_FROM_VARIABLE": generateAssignmentVariableCode,
         "STATEMENT_ASSIGNMENT_VARIABLE_FROM_TEMP_VARIABLE": generateAssignmentVariableCode,
@@ -926,7 +1017,7 @@ setStatementDispatchDict(
         "STATEMENT_SPECIAL_UNPACK_CHECK_FROM_ITERATED": generateUnpackCheckFromIteratedCode,
         "STATEMENT_EXEC": generateExecCode,
         "STATEMENT_LOCALS_DICT_SYNC": generateLocalsDictSyncCode,
-        "STATEMENT_SET_LOCALS": generateSetLocalsDictCode,
+        "STATEMENT_SET_LOCALS": generateSetLocalsMappingCode,
         "STATEMENT_SET_LOCALS_DICTIONARY": generateSetLocalsDictCode,
         "STATEMENT_RELEASE_LOCALS": generateReleaseLocalsDictCode,
         "STATEMENT_PRESERVE_FRAME_EXCEPTION": generateFramePreserveExceptionCode,
