@@ -1,4 +1,4 @@
-#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2023, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -18,7 +18,6 @@
 """Code generation for package resources access."""
 
 from nuitka.Options import shallMakeModule
-from nuitka.utils.Distributions import getDistributionTopLevelPackageNames
 
 from .CallCodes import (
     getCallCodeKwPairs,
@@ -31,6 +30,7 @@ from .CodeHelpers import (
     generateChildExpressionsCode,
     withObjectCodeTemporaryAssignment,
 )
+from .ConstantCodes import addDistributionMetadataValue
 from .DictCodes import getDictionaryCreationCode
 from .ErrorCodes import getErrorExitCode
 from .ImportCodes import getImportModuleNameHardCode
@@ -165,23 +165,17 @@ def generateImportlibMetadataBackportEntryPointsCallCode(
 def generateImportlibMetadataDistributionValueCode(to_name, expression, emit, context):
     distribution = expression.distribution
     original_name = expression.original_name
-    metadata = (
-        distribution.read_text("METADATA") or distribution.read_text("METADATA") or ""
-    )
 
-    metadata = str(metadata)
+    addDistributionMetadataValue(name=original_name, distribution=distribution)
 
     with withObjectCodeTemporaryAssignment(
         to_name, "distribution_value", expression, emit, context
     ) as value_name:
-
         emit(
-            """%(to_name)s = Nuitka_Distribution_New("%(name)s", "%(package_name)s", %(metadata)s);"""
+            """%(to_name)s = Nuitka_Distribution_New(%(name)s);"""
             % {
                 "to_name": value_name,
-                "name": original_name,
-                "package_name": getDistributionTopLevelPackageNames(distribution)[0],
-                "metadata": context.getConstantCode(constant=metadata),
+                "name": context.getConstantCode(original_name),
             }
         )
 
@@ -838,6 +832,32 @@ def generateOsPathBasenameCallCode(to_name, expression, emit, context):
     )
 
 
+def generateOsPathAbspathCallCode(to_name, expression, emit, context):
+    generateCAPIObjectCode(
+        to_name=to_name,
+        capi="OS_PATH_ABSPATH",
+        arg_desc=(("path_arg", expression.subnode_path),),
+        may_raise=expression.mayRaiseException(BaseException),
+        conversion_check=decideConversionCheckNeeded(to_name, expression),
+        source_ref=expression.getCompatibleSourceReference(),
+        emit=emit,
+        context=context,
+    )
+
+
+def generateOsPathIsabsCallCode(to_name, expression, emit, context):
+    generateCAPIObjectCode(
+        to_name=to_name,
+        capi="OS_PATH_ISABS",
+        arg_desc=(("path_arg", expression.subnode_s),),
+        may_raise=expression.mayRaiseException(BaseException),
+        conversion_check=decideConversionCheckNeeded(to_name, expression),
+        source_ref=expression.getCompatibleSourceReference(),
+        emit=emit,
+        context=context,
+    )
+
+
 def generateOsListdirCallCode(to_name, expression, emit, context):
     generateCAPIObjectCode(
         to_name=to_name,
