@@ -1,4 +1,4 @@
-#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2023, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -31,10 +31,12 @@ from nuitka.containers.OrderedSets import OrderedSet
 from nuitka.importing.Importing import (
     addMainScriptDirectory,
     decideModuleSourceRef,
+    flushImportCache,
     locateModule,
 )
 from nuitka.Tracing import wheel_logger
 from nuitka.utils.Execution import check_call
+from nuitka.utils.FileOperations import deleteFile, renameFile
 from nuitka.utils.ModuleNames import ModuleName
 
 
@@ -112,7 +114,7 @@ class build(distutils.command.build.build):
 
         self._build(os.path.abspath(self.build_lib))
 
-    def _findBuildTasks(self):
+    def _findBuildTasks2(self):
         """
         Helper for _build
         Returns list containing bool (is_package) and module_names
@@ -138,6 +140,12 @@ class build(distutils.command.build.build):
                 module_name=script_module_name, parent_package=None, level=0
             )[1]
 
+            if script_module_filename is None:
+                wheel_logger.sysexit(
+                    "Error, failed to locate script containing module '%s'"
+                    % script_module_name
+                )
+
             # Decide package or module.
             (
                 _main_added,
@@ -175,6 +183,42 @@ class build(distutils.command.build.build):
 
         return builds
 
+    def _findBuildTasks(self):
+        builds = self._findBuildTasks2()
+        result = []
+
+        for _is_package, module_name_orig in builds:
+            _module_name, main_filename, module_kind, finding = locateModule(
+                module_name=module_name_orig,
+                parent_package=None,
+                level=0,
+            )
+
+            # Handle extension modules already compiled. They are either to be replaced, or
+            # they are included as they are, because there is no source, then the task can
+            # be skipped.
+            if module_kind == "extension":
+                main_filename_away = main_filename + ".away"
+                renameFile(main_filename, main_filename_away)
+
+                flushImportCache()
+
+                _module_name, main_filename, module_kind, finding = locateModule(
+                    module_name=module_name_orig,
+                    parent_package=None,
+                    level=0,
+                )
+
+                if finding != "not-found":
+                    deleteFile(main_filename_away, must_exist=True)
+                else:
+                    renameFile(main_filename_away, main_filename)
+                    continue
+
+            result.append((_is_package, module_name_orig))
+
+        return result
+
     @staticmethod
     def _parseOptionsEntry(option, value):
         option = "--" + option.lstrip("-")
@@ -197,21 +241,18 @@ class build(distutils.command.build.build):
     def _build(self, build_lib):
         # High complexity, pylint: disable=too-many-branches,too-many-locals
 
-        # Remove Python code from build folder, that's our job now.
-        for root, _, filenames in os.walk(build_lib):
-            for filename in filenames:
-                fullpath = os.path.join(root, filename)
-
-                if fullpath.lower().endswith((".py", ".pyw", ".pyc", ".pyo")):
-                    os.unlink(fullpath)
-
         old_dir = os.getcwd()
+
+        # Let's use the python source files in the build_lib since these should
+        # get copied over.
         os.chdir(build_lib)
 
         if self.distribution.package_dir and "" in self.distribution.package_dir:
-            main_package_dir = self.distribution.package_dir.get("")
+            main_package_dir = os.path.join(
+                build_lib, self.distribution.package_dir.get("")
+            )
         else:
-            main_package_dir = os.path.abspath(old_dir)
+            main_package_dir = os.path.abspath(build_lib)
 
         # Search in the build directory preferably.
         addMainScriptDirectory(main_package_dir)
@@ -300,6 +341,14 @@ class build(distutils.command.build.build):
 
         os.chdir(old_dir)
 
+        # Remove Python code from build folder, that's our job now.
+        for root, _, filenames in os.walk(build_lib):
+            for filename in filenames:
+                fullpath = os.path.join(root, filename)
+
+                if fullpath.lower().endswith((".py", ".pyw", ".pyc", ".pyo")):
+                    os.unlink(fullpath)
+
 
 # Required by distutils, used as command name, pylint: disable=invalid-name
 class install(distutils.command.install.install):
