@@ -1,4 +1,4 @@
-#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2023, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -32,6 +32,7 @@ from nuitka.Bytecodes import compileSourceToBytecode
 from nuitka.containers.OrderedSets import OrderedSet
 from nuitka.Errors import NuitkaForbiddenDLLEncounter
 from nuitka.importing import ImportCache
+from nuitka.importing.Importing import getPythonUnpackedSearchPath
 from nuitka.importing.StandardLibrary import (
     getStandardLibraryPaths,
     isStandardLibraryNoAutoInclusionModule,
@@ -50,9 +51,15 @@ from nuitka.Progress import (
 )
 from nuitka.PythonVersions import python_version
 from nuitka.Tracing import general, inclusion_logger, printError
-from nuitka.tree.SourceHandling import readSourceCodeFromFilename
+from nuitka.tree.SourceHandling import (
+    readSourceCodeFromFilenameWithInformation,
+)
 from nuitka.utils.Execution import executeProcess
-from nuitka.utils.FileOperations import areSamePaths
+from nuitka.utils.FileOperations import (
+    areInSamePaths,
+    areSamePaths,
+    isFilenameBelowPath,
+)
 from nuitka.utils.ModuleNames import ModuleName
 from nuitka.utils.SharedLibraries import copyDllFile, setSharedLibraryRPATH
 from nuitka.utils.Signing import addMacOSCodeSignature
@@ -130,7 +137,11 @@ def _detectedSourceFile(filename, module_name, result, user_provided, technical)
             technical=technical,
         )
 
-    source_code = readSourceCodeFromFilename(module_name, filename)
+    (
+        source_code,
+        _original_code,
+        _contributing_plugins,
+    ) = readSourceCodeFromFilenameWithInformation(module_name, filename)
 
     if module_name == "site":
         if source_code.startswith("def ") or source_code.startswith("class "):
@@ -155,11 +166,12 @@ __file__ = (__nuitka_binary_dir + '%s%s') if '__nuitka_binary_dir' in dict(__bui
 
     is_package = os.path.basename(filename) == "__init__.py"
 
-    # Plugins can modify source code:
+    # Plugins can modify source code again, this time knowing it will be frozen.
     source_code = Plugins.onFrozenModuleSourceCode(
         module_name=module_name, is_package=is_package, source_code=source_code
     )
 
+    # TODO: Not yet handling SyntaxError here, although plugins might cause them.
     bytecode = compileSourceToBytecode(
         source_code=source_code,
         filename=module_name.replace(".", os.path.sep) + ".py",
@@ -504,15 +516,17 @@ def checkFreezingModuleSet():
     problem_modules = OrderedSet()
 
     if isDebianBasedLinux():
-        message = (
-            "Standard with Python package from Debian installation may not be working."
-        )
+        message = "Standalone with Python package from Debian installation may not be working."
         mnemonic = "debian-dist-packages"
 
         def checkModulePath(module):
-            if "dist-packages" in module.getCompileTimeFilename().split("/"):
-                module_name = module.getFullName()
+            module_filename = module.getCompileTimeFilename()
+            module_name = module.getFullName()
 
+            if (
+                "dist-packages" in module_filename.split("/")
+                and not module_name.isFakeModuleName()
+            ):
                 package_name = module_name.getTopLevelPackageName()
 
                 if package_name is not None:
@@ -522,7 +536,11 @@ def checkFreezingModuleSet():
 
     else:
         checkModulePath = None
+        message = None
+        mnemonic = None
 
+    # We intend for other platforms to join, e.g. Fedora, etc. but currently
+    # only Debian is done.
     if checkModulePath is not None:
         for module in ModuleRegistry.getDoneModules():
             checkModulePath(module)
@@ -603,6 +621,23 @@ def copyDllsUsed(dist_dir, standalone_entry_points):
         for standalone_entry_point in standalone_entry_points[1:]
         if not standalone_entry_point.kind.endswith("_ignored")
     ]
+    main_standalone_entry_point = standalone_entry_points[0]
+
+    if isMacOS():
+        fixupBinaryDLLPathsMacOS(
+            binary_filename=os.path.join(
+                dist_dir, main_standalone_entry_point.dest_path
+            ),
+            package_name=main_standalone_entry_point.package_name,
+            original_location=main_standalone_entry_point.source_path,
+            standalone_entry_points=standalone_entry_points,
+        )
+
+        # After dependency detection, we can change the RPATH for macOS main
+        # binary.
+        setSharedLibraryRPATH(
+            os.path.join(dist_dir, standalone_entry_points[0].dest_path), "$ORIGIN"
+        )
 
     setupProgressBar(
         stage="Copying used DLLs",
@@ -632,18 +667,13 @@ def copyDllsUsed(dist_dir, standalone_entry_points):
 
     closeProgressBar()
 
-    # After dependency detection, we can change the RPATH for macOS main binary.
-    if isMacOS():
-        setSharedLibraryRPATH(
-            os.path.join(dist_dir, standalone_entry_points[0].dest_path), "$ORIGIN"
-        )
-
     # Add macOS code signature
     if isMacOS():
         addMacOSCodeSignature(
             filenames=[
                 os.path.join(dist_dir, standalone_entry_point.dest_path)
-                for standalone_entry_point in copy_standalone_entry_points
+                for standalone_entry_point in [main_standalone_entry_point]
+                + copy_standalone_entry_points
             ]
         )
 
@@ -652,10 +682,6 @@ def copyDllsUsed(dist_dir, standalone_entry_points):
     )
 
 
-def _decideTargetPath(dll_filename):
-    return os.path.basename(dll_filename)
-
-
 def _detectUsedDLLs(standalone_entry_point, source_dir):
     binary_filename = standalone_entry_point.source_path
     try:
@@ -671,21 +697,61 @@ def _detectUsedDLLs(standalone_entry_point, source_dir):
     except NuitkaForbiddenDLLEncounter:
         inclusion_logger.info("Not including forbidden DLL '%s'." % binary_filename)
     else:
-        # Allow plugins can prevent inclusion, this may discard things from used_dlls.
-        Plugins.removeDllDependencies(
-            dll_filename=binary_filename, dll_filenames=used_dlls
+        # Plugins generally decide if they allow dependencies from the outside
+        # based on the package name.
+        allow_outside_dependencies = Plugins.decideAllowOutsideDependencies(
+            standalone_entry_point.package_name
         )
 
-        for used_dll in used_dlls:
-            addIncludedEntryPoint(
-                makeDllEntryPoint(
-                    logger=inclusion_logger,
-                    source_path=used_dll,
-                    dest_path=_decideTargetPath(used_dll),
-                    package_name=standalone_entry_point.package_name,
+        # TODO: Command line option maybe
+        if allow_outside_dependencies is None:
+            allow_outside_dependencies = True
+
+        if not allow_outside_dependencies:
+            inside_paths = getPythonUnpackedSearchPath()
+
+            def decideInside(dll_filename):
+                return any(
+                    isFilenameBelowPath(path=inside_path, filename=dll_filename)
+                    for inside_path in inside_paths
                 )
+
+            used_dlls = set(
+                dll_filename for dll_filename in used_dlls if decideInside(dll_filename)
             )
 
+        # Allow plugins can prevent inclusion, this may discard things from used_dlls.
+        removed_dlls = Plugins.removeDllDependencies(
+            dll_filename=binary_filename, dll_filenames=used_dlls
+        )
+        used_dlls = tuple(OrderedSet(used_dlls) - OrderedSet(removed_dlls))
+
+        for used_dll in used_dlls:
+            # TODO: If used by a DLL from the same folder, put it there,
+            # otherwise top level, but for now this is limited to the case where
+            # it is required that way only, because it broke other things.
+            if standalone_entry_point.package_name == "openvino" and areInSamePaths(
+                standalone_entry_point.source_path, used_dll
+            ):
+                dest_path = os.path.normpath(
+                    os.path.join(
+                        os.path.dirname(standalone_entry_point.dest_path),
+                        os.path.basename(used_dll),
+                    )
+                )
+            else:
+                dest_path = os.path.basename(used_dll)
+
+            dll_entry_point = makeDllEntryPoint(
+                logger=inclusion_logger,
+                source_path=used_dll,
+                dest_path=dest_path,
+                package_name=standalone_entry_point.package_name,
+                reason="Used by '%s'" % standalone_entry_point.dest_path,
+            )
+
+            addIncludedEntryPoint(dll_entry_point)
+
 
 def detectUsedDLLs(standalone_entry_points, source_dir):
     setupProgressBar(
