@@ -271,10 +271,12 @@ _stdlib_no_auto_inclusion_list = (
     "decimal",
     "_pydecimal",
     "_decimal",
+    "statistics",
     "lzma",
     "_lzma",
     "bz2",
     "_bz2",
+    "logging",
     "subprocess",
     "socket",
     "selectors",
