@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2023, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -28,28 +28,30 @@ import math
 from nuitka.PythonVersions import python_version
 from nuitka.specs import BuiltinParameterSpecs
 
-from .ExpressionBases import (
-    ExpressionChildHavingBase,
-    ExpressionChildrenHavingBase,
+from .ChildrenHavingMixins import (
+    ChildHavingLowMixin,
+    ChildrenHavingLowHighMixin,
+    ChildrenHavingLowHighStepMixin,
 )
+from .ExpressionBases import ExpressionBase
+from .ExpressionShapeMixins import ExpressionListShapeExactMixin
 from .IterationHandles import (
     IterationHandleRange1,
     IterationHandleRange2,
     IterationHandleRange3,
 )
 from .NodeMakingHelpers import makeConstantReplacementNode
-from .shapes.BuiltinTypeShapes import tshape_list, tshape_xrange
+from .shapes.BuiltinTypeShapes import tshape_xrange
 
 
-class ExpressionBuiltinRangeMixin(object):
-    """ Mixin class for range nodes with 1/2/3 arguments. """
+class ExpressionBuiltinRangeMixin(ExpressionListShapeExactMixin):
+    """Mixin class for range nodes with 1/2/3 arguments."""
+
+    # Mixins are required to slots
+    __slots__ = ()
 
     builtin_spec = BuiltinParameterSpecs.builtin_range_spec
 
-    @staticmethod
-    def getTypeShape():
-        return tshape_list
-
     def getTruthValue(self):
         length = self.getIterationLength()
 
@@ -111,46 +113,45 @@ class ExpressionBuiltinRangeMixin(object):
     def computeExpressionIter1(self, iter_node, trace_collection):
         assert python_version < 0x300
 
-        iteration_length = self.getIterationLength()
+        # TODO: The xrange is always faster and more memory usage than range, so this makes no sense, to
+        # use it as a source for any iteration, esp. as xrange is the Python3 only type that will be
+        # best optimized.
+        result = makeExpressionBuiltinXrange(
+            low=self.subnode_low,
+            high=self.subnode_high,
+            step=self.subnode_step,
+            source_ref=self.source_ref,
+        )
 
-        if iteration_length is not None and iteration_length > 256:
-            result = makeExpressionBuiltinXrange(
-                low=self.subnode_low,
-                high=self.subnode_high,
-                step=self.subnode_step,
-                source_ref=self.source_ref,
-            )
+        self.parent.replaceChild(self, result)
+        del self.parent
 
-            self.parent.replaceChild(self, result)
-            del self.parent
-
-            return (
-                iter_node,
-                "new_expression",
-                "Replaced 'range' with 'xrange' built-in call for iteration.",
-            )
-
-        # No exception will be raised on ranges.
-
-        return iter_node, None, None
+        return (
+            iter_node,
+            "new_expression",
+            "Replaced 'range' with 'xrange' built-in call for iteration.",
+        )
 
     def canPredictIterationValues(self):
         return self.getIterationLength() is not None
 
 
-class ExpressionBuiltinRange1(ExpressionBuiltinRangeMixin, ExpressionChildHavingBase):
+class ExpressionBuiltinRange1(
+    ExpressionBuiltinRangeMixin, ChildHavingLowMixin, ExpressionBase
+):
     kind = "EXPRESSION_BUILTIN_RANGE1"
 
-    named_child = "low"
+    python_version_spec = "< 0x300"
+
+    named_children = ("low",)
 
     subnode_high = None
     subnode_step = None
 
     def __init__(self, low, source_ref):
-        assert low is not None
-        assert python_version < 0x300
+        ChildHavingLowMixin.__init__(self, low=low)
 
-        ExpressionChildHavingBase.__init__(self, value=low, source_ref=source_ref)
+        ExpressionBase.__init__(self, source_ref)
 
     def computeExpression(self, trace_collection):
         low = self.subnode_low
@@ -194,19 +195,26 @@ class ExpressionBuiltinRange1(ExpressionBuiltinRangeMixin, ExpressionChildHaving
 
 
 class ExpressionBuiltinRange2(
-    ExpressionBuiltinRangeMixin, ExpressionChildrenHavingBase
+    ExpressionBuiltinRangeMixin, ChildrenHavingLowHighMixin, ExpressionBase
 ):
     kind = "EXPRESSION_BUILTIN_RANGE2"
 
+    python_version_spec = "< 0x300"
+
     named_children = ("low", "high")
 
+    # For the mixing to work generically
     subnode_step = None
 
     def __init__(self, low, high, source_ref):
-        ExpressionChildrenHavingBase.__init__(
-            self, values={"low": low, "high": high}, source_ref=source_ref
+        ChildrenHavingLowHighMixin.__init__(
+            self,
+            low=low,
+            high=high,
         )
 
+        ExpressionBase.__init__(self, source_ref)
+
     builtin_spec = BuiltinParameterSpecs.builtin_range_spec
 
     def computeExpression(self, trace_collection):
@@ -274,19 +282,24 @@ class ExpressionBuiltinRange2(
 
 
 class ExpressionBuiltinRange3(
-    ExpressionBuiltinRangeMixin, ExpressionChildrenHavingBase
+    ExpressionBuiltinRangeMixin, ChildrenHavingLowHighStepMixin, ExpressionBase
 ):
     kind = "EXPRESSION_BUILTIN_RANGE3"
 
+    python_version_spec = "< 0x300"
+
     named_children = ("low", "high", "step")
 
     def __init__(self, low, high, step, source_ref):
-        assert python_version < 0x300
-
-        ExpressionChildrenHavingBase.__init__(
-            self, values={"low": low, "high": high, "step": step}, source_ref=source_ref
+        ChildrenHavingLowHighStepMixin.__init__(
+            self,
+            low=low,
+            high=high,
+            step=step,
         )
 
+        ExpressionBase.__init__(self, source_ref)
+
     builtin_spec = BuiltinParameterSpecs.builtin_range_spec
 
     def computeExpression(self, trace_collection):
@@ -388,7 +401,10 @@ class ExpressionBuiltinRange3(
 
 
 class ExpressionBuiltinXrangeMixin(object):
-    """ Mixin class for xrange nodes with 1/2/3 arguments. """
+    """Mixin class for xrange nodes with 1/2/3 arguments."""
+
+    # Mixins are required to slots
+    __slots__ = ()
 
     builtin_spec = BuiltinParameterSpecs.builtin_xrange_spec
 
@@ -458,16 +474,20 @@ class ExpressionBuiltinXrangeMixin(object):
         return iter_node, None, None
 
 
-class ExpressionBuiltinXrange1(ExpressionBuiltinXrangeMixin, ExpressionChildHavingBase):
+class ExpressionBuiltinXrange1(
+    ExpressionBuiltinXrangeMixin, ChildHavingLowMixin, ExpressionBase
+):
     kind = "EXPRESSION_BUILTIN_XRANGE1"
 
-    named_child = "low"
+    named_children = ("low",)
 
     subnode_high = None
     subnode_step = None
 
     def __init__(self, low, source_ref):
-        ExpressionChildHavingBase.__init__(self, value=low, source_ref=source_ref)
+        ChildHavingLowMixin.__init__(self, low=low)
+
+        ExpressionBase.__init__(self, source_ref)
 
     def computeExpression(self, trace_collection):
         low = self.subnode_low
@@ -502,7 +522,7 @@ class ExpressionBuiltinXrange1(ExpressionBuiltinXrangeMixin, ExpressionChildHavi
 
 
 class ExpressionBuiltinXrange2(
-    ExpressionBuiltinXrangeMixin, ExpressionChildrenHavingBase
+    ExpressionBuiltinXrangeMixin, ChildrenHavingLowHighMixin, ExpressionBase
 ):
     kind = "EXPRESSION_BUILTIN_XRANGE2"
 
@@ -511,10 +531,14 @@ class ExpressionBuiltinXrange2(
     subnode_step = None
 
     def __init__(self, low, high, source_ref):
-        ExpressionChildrenHavingBase.__init__(
-            self, values={"low": low, "high": high}, source_ref=source_ref
+        ChildrenHavingLowHighMixin.__init__(
+            self,
+            low=low,
+            high=high,
         )
 
+        ExpressionBase.__init__(self, source_ref)
+
     def computeExpression(self, trace_collection):
         low = self.subnode_low
         high = self.subnode_high
@@ -564,17 +588,22 @@ class ExpressionBuiltinXrange2(
 
 
 class ExpressionBuiltinXrange3(
-    ExpressionBuiltinXrangeMixin, ExpressionChildrenHavingBase
+    ExpressionBuiltinXrangeMixin, ChildrenHavingLowHighStepMixin, ExpressionBase
 ):
     kind = "EXPRESSION_BUILTIN_XRANGE3"
 
     named_children = ("low", "high", "step")
 
     def __init__(self, low, high, step, source_ref):
-        ExpressionChildrenHavingBase.__init__(
-            self, values={"low": low, "high": high, "step": step}, source_ref=source_ref
+        ChildrenHavingLowHighStepMixin.__init__(
+            self,
+            low=low,
+            high=high,
+            step=step,
         )
 
+        ExpressionBase.__init__(self, source_ref)
+
     def computeExpression(self, trace_collection):
         low = self.subnode_low
         high = self.subnode_high
