@@ -1,4 +1,4 @@
-#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2023, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -27,6 +27,9 @@ from abc import abstractmethod
 
 from nuitka import Options
 from nuitka.__past__ import long
+
+# TODO: Probably should separate building reports out.
+from nuitka.code_generation.Reports import onMissingOverload
 from nuitka.Constants import isCompileTimeConstantValue
 from nuitka.PythonVersions import python_version
 
@@ -868,6 +871,22 @@ class ExpressionBase(NodeBase):
         """
         return None
 
+    def getExpressionDictInConstant(self, value):
+        """Value that the dict "in" operation would give, if known.
+
+        This is only called for values with known dict type shape. And those
+        nodes who are known to do it, have to overload it.
+        """
+
+        # Virtual method, pylint: disable=unused-argument
+
+        # We want to have them all overloaded, so lets report cases where that
+        # has not been happening.
+        if Options.is_debug:
+            onMissingOverload(method_name="getExpressionDictInConstant", node=self)
+
+        return None
+
     def hasShapeTrustedAttributes(self):
         return self.getTypeShape().hasShapeTrustedAttributes()
 
