@@ -1,4 +1,4 @@
-#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2023, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -157,7 +157,7 @@ class ExpressionFunctionBodyBase(
         current = self
 
         while not current.isCompiledPythonModule():
-            if current.isExpressionClassBody():
+            if current.isExpressionClassBodyBase():
                 return current
 
             current = current.getParentVariableProvider()
@@ -293,7 +293,7 @@ class ExpressionFunctionBodyBase(
             # dict access.
             if (
                 python_version < 0x300
-                and not entry_point.isExpressionClassBody()
+                and not entry_point.isExpressionClassBodyBase()
                 and not entry_point.isPythonMainModule()
                 and result.isModuleVariable()
                 and entry_point.isUnoptimized()
@@ -568,9 +568,11 @@ class ExpressionFunctionBody(
     MarkUnoptimizedFunctionIndicatorMixin,
     ExpressionFunctionEntryPointBase,
 ):
+    # TODO: There should be more special ones than this general type in order to
+    # not cover exec ones in the same object. pylint: disable=too-many-instance-attributes
+
     kind = "EXPRESSION_FUNCTION_BODY"
 
-    # TODO: These should be more special than the general type in order to not cover exec ones.
     __slots__ = (
         "unoptimized_locals",
         "unqualified_exec",
@@ -1009,8 +1011,7 @@ class ExpressionFunctionCreationMixin(SideEffectsFromChildrenMixin):
                 result,
                 "new_statements",  # TODO: More appropriate tag maybe.
                 """\
-Replaced call to created function body '%s' with direct \
-function call."""
+Replaced call to created function body '%s' with direct function call."""
                 % self.getName(),
             )
 
