@@ -1,4 +1,4 @@
-#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2023, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -26,28 +26,36 @@ from nuitka.optimizations.TraceCollections import TraceCollectionBranch
 from nuitka.PythonVersions import python_version
 from nuitka.tree.TreeHelpers import makeStatementsSequence
 
+from .ChildrenHavingMixins import ChildHavingFallbackMixin
 from .ConditionalNodes import ExpressionConditional
-from .ConstantRefNodes import ExpressionConstantDictEmptyRef
-from .ExpressionBases import ExpressionBase, ExpressionChildHavingBase
-from .NodeBases import StatementBase, StatementChildHavingBase
+from .ExpressionBases import ExpressionBase
+from .GlobalsLocalsNodes import ExpressionBuiltinLocalsRef
+from .NodeBases import StatementBase
+from .StatementBasesGenerated import (
+    StatementLocalsDictOperationSetBase,
+    StatementSetLocalsBase,
+    StatementSetLocalsDictionaryBase,
+)
 from .VariableAssignNodes import makeStatementAssignmentVariable
 from .VariableDelNodes import makeStatementDelVariable
 from .VariableRefNodes import ExpressionTempVariableRef
 from .VariableReleaseNodes import makeStatementReleaseVariable
 
 
-class ExpressionLocalsVariableRefOrFallback(ExpressionChildHavingBase):
+class ExpressionLocalsVariableRefOrFallback(ChildHavingFallbackMixin, ExpressionBase):
     kind = "EXPRESSION_LOCALS_VARIABLE_REF_OR_FALLBACK"
 
-    named_child = "fallback"
+    named_children = ("fallback",)
 
     __slots__ = ("locals_scope", "variable", "variable_trace")
 
     def __init__(self, locals_scope, variable_name, fallback, source_ref):
-        ExpressionChildHavingBase.__init__(self, value=fallback, source_ref=source_ref)
-
         assert locals_scope is not None
 
+        ChildHavingFallbackMixin.__init__(self, fallback=fallback)
+
+        ExpressionBase.__init__(self, source_ref)
+
         self.locals_scope = locals_scope
         self.variable = locals_scope.getLocalsDictVariable(variable_name)
 
@@ -86,9 +94,13 @@ class ExpressionLocalsVariableRefOrFallback(ExpressionChildHavingBase):
             # Need to compute the replacement still.
             return replacement.computeExpressionRaw(trace_collection)
 
-        no_exec = not self.locals_scope.isUnoptimizedFunctionScope()
+        # TODO: Split exec locals variable references out to distinct node type.
+        no_exec = (
+            not self.locals_scope.isUnoptimizedFunctionScope()
+            and not self.locals_scope.isPreventedPropagation()
+        )
 
-        # if we can be sure if doesn't have a value set, go to the fallback directly.
+        # If we can be sure it doesn't have a value set, go to the fallback directly.
         if no_exec and self.variable_trace.mustNotHaveValue():
             return trace_collection.computedExpressionResultRaw(
                 self.subnode_fallback,
@@ -113,18 +125,35 @@ class ExpressionLocalsVariableRefOrFallback(ExpressionChildHavingBase):
 
             # Need to compute the replacement still.
             return result.computeExpressionRaw(trace_collection)
-        else:
-            trace_collection.onExceptionRaiseExit(BaseException)
 
-            branch_fallback = TraceCollectionBranch(
-                parent=trace_collection, name="fallback node usage"
-            )
+        trace_collection.onExceptionRaiseExit(BaseException)
 
-            branch_fallback.computeBranch(self.subnode_fallback)
+        branch_fallback = TraceCollectionBranch(
+            parent=trace_collection, name="fallback node usage"
+        )
 
-            trace_collection.mergeBranches(branch_fallback, None)
+        if (
+            self.variable_trace.isUnknownTrace()
+            and self.subnode_fallback.isExpressionVariableRef()
+        ):
+            fallback_variable_trace = self.subnode_fallback.variable_trace
 
-            return self, None, None
+            if fallback_variable_trace is not None:
+                trusted_node = (
+                    self.subnode_fallback.variable_trace.getAttributeNodeVeryTrusted()
+                )
+
+                if trusted_node is not None:
+                    return trace_collection.computedExpressionResultRaw(
+                        expression=self.subnode_fallback,
+                        change_tags="var_usage",
+                        change_desc="Hard value referenced in class not considering class dictionary.",
+                    )
+
+        branch_fallback.onExpression(self.subnode_fallback)
+        trace_collection.mergeBranches(branch_fallback, None)
+
+        return self, None, None
 
     def computeExpressionCall(self, call_node, call_args, call_kw, trace_collection):
         trace_collection.onExceptionRaiseExit(BaseException)
@@ -148,18 +177,17 @@ class ExpressionLocalsVariableRefOrFallback(ExpressionChildHavingBase):
 
             # Create a cloned node with the locals variable.
             call_node_clone = call_node.makeClone()
-            call_node_clone.setChild(
-                "called",
+            call_node_clone.setChildCalled(
                 ExpressionLocalsVariableRef(
                     locals_scope=self.locals_scope,
                     variable_name=variable_name,
                     source_ref=self.source_ref,
-                ),
+                )
             )
 
             # Make the original one for the fallback
             call_node = call_node.makeCloneShallow()
-            call_node.setChild("called", self.subnode_fallback)
+            call_node.setChildCalled(self.subnode_fallback)
 
             result = ExpressionConditional(
                 condition=ExpressionLocalsVariableCheck(
@@ -204,7 +232,7 @@ class ExpressionLocalsVariableRef(ExpressionBase):
 
     def __init__(self, locals_scope, variable_name, source_ref):
 
-        ExpressionBase.__init__(self, source_ref=source_ref)
+        ExpressionBase.__init__(self, source_ref)
 
         self.locals_scope = locals_scope
         self.variable = locals_scope.getLocalsDictVariable(variable_name)
@@ -223,7 +251,10 @@ class ExpressionLocalsVariableRef(ExpressionBase):
         }
 
     def getDetailsForDisplay(self):
-        return {"variable_name": self.getVariableName()}
+        return {
+            "locals_scope": self.locals_scope.getCodeName(),
+            "variable_name": self.getVariableName(),
+        }
 
     def getVariableName(self):
         return self.variable.getName()
@@ -300,7 +331,7 @@ class ExpressionLocalsVariableCheck(ExpressionBase):
     def __init__(self, locals_scope, variable_name, source_ref):
         self.variable_name = variable_name
 
-        ExpressionBase.__init__(self, source_ref=source_ref)
+        ExpressionBase.__init__(self, source_ref)
 
         self.locals_scope = locals_scope
 
@@ -322,31 +353,25 @@ class ExpressionLocalsVariableCheck(ExpressionBase):
         return self, None, None
 
 
-class StatementLocalsDictOperationSet(StatementChildHavingBase):
+class StatementLocalsDictOperationSet(StatementLocalsDictOperationSetBase):
     kind = "STATEMENT_LOCALS_DICT_OPERATION_SET"
 
-    named_child = "source"
+    named_children = ("source",)
+    node_attributes = ("locals_scope", "variable_name")
+    auto_compute_handling = "post_init"
 
-    __slots__ = ("variable", "variable_version", "variable_trace", "locals_scope")
+    __slots__ = ("variable", "variable_version", "variable_trace")
 
-    # TODO: Specialize for Python3 maybe to save attribute for Python2.
-    may_raise_set = python_version >= 0x300
+    # false alarm due to post_init, pylint: disable=attribute-defined-outside-init
 
-    def __init__(self, locals_scope, variable_name, value, source_ref):
-        assert type(variable_name) is str
-        assert value is not None
-
-        StatementChildHavingBase.__init__(self, value=value, source_ref=source_ref)
-
-        assert locals_scope is not None
-
-        self.variable = locals_scope.getLocalsDictVariable(variable_name=variable_name)
+    def postInitNode(self):
 
+        self.variable = self.locals_scope.getLocalsDictVariable(
+            variable_name=self.variable_name
+        )
         self.variable_version = self.variable.allocateTargetNumber()
         self.variable_trace = None
 
-        self.locals_scope = locals_scope
-
     def finalize(self):
         del self.parent
         del self.locals_scope
@@ -358,6 +383,12 @@ class StatementLocalsDictOperationSet(StatementChildHavingBase):
             "variable_name": self.getVariableName(),
         }
 
+    def getDetailsForDisplay(self):
+        return {
+            "locals_scope": self.locals_scope.getCodeName(),
+            "variable_name": self.getVariableName(),
+        }
+
     def getVariableName(self):
         return self.variable.getName()
 
@@ -369,10 +400,8 @@ class StatementLocalsDictOperationSet(StatementChildHavingBase):
 
     def computeStatement(self, trace_collection):
         if self.locals_scope.isMarkedForPropagation():
-            variable_name = self.getVariableName()
-
             variable = self.locals_scope.allocateTempReplacementVariable(
-                trace_collection=trace_collection, variable_name=variable_name
+                trace_collection=trace_collection, variable_name=self.variable_name
             )
 
             result = makeStatementAssignmentVariable(
@@ -405,15 +434,30 @@ class StatementLocalsDictOperationSet(StatementChildHavingBase):
             variable=self.variable, version=self.variable_version, assign_node=self
         )
 
-        if self.may_raise_set:
+        if not self.locals_scope.hasShapeDictionaryExact():
             trace_collection.onExceptionRaiseExit(BaseException)
 
         return self, None, None
 
-    def mayRaiseException(self, exception_type):
-        return self.may_raise_set or self.subnode_source.mayRaiseException(
-            exception_type
-        )
+    if python_version >= 0x300:
+
+        def mayRaiseException(self, exception_type):
+            return (
+                not self.locals_scope.hasShapeDictionaryExact()
+                or self.subnode_source.mayRaiseException(exception_type)
+            )
+
+        def mayRaiseExceptionOperation(self):
+            return not self.locals_scope.hasShapeDictionaryExact()
+
+    else:
+
+        def mayRaiseException(self, exception_type):
+            return self.subnode_source.mayRaiseException(exception_type)
+
+        @staticmethod
+        def mayRaiseExceptionOperation():
+            return False
 
     @staticmethod
     def getStatementNiceName():
@@ -458,8 +502,14 @@ class StatementLocalsDictOperationDel(StatementBase):
 
     def getDetails(self):
         return {
-            "variable_name": self.getVariableName(),
             "locals_scope": self.locals_scope,
+            "variable_name": self.getVariableName(),
+        }
+
+    def getDetailsForDisplay(self):
+        return {
+            "locals_scope": self.locals_scope.getCodeName(),
+            "variable_name": self.getVariableName(),
         }
 
     def getVariableName(self):
@@ -522,28 +572,27 @@ class StatementLocalsDictOperationDel(StatementBase):
         return "locals dictionary value del statement"
 
 
-class StatementSetLocals(StatementChildHavingBase):
-    kind = "STATEMENT_SET_LOCALS"
-
-    named_child = "new_locals"
-
-    __slots__ = ("locals_scope",)
-
-    def __init__(self, locals_scope, new_locals, source_ref):
-        StatementChildHavingBase.__init__(self, value=new_locals, source_ref=source_ref)
-
-        self.locals_scope = locals_scope
-
-    def finalize(self):
-        del self.parent
-        del self.locals_scope
-        del self.subnode_new_locals
+class StatementSetLocalsMixin(object):
+    __slots__ = ()
 
     def getDetailsForDisplay(self):
         return {"locals_scope": self.locals_scope.getCodeName()}
 
-    def getDetails(self):
-        return {"locals_scope": self.locals_scope}
+    def getLocalsScope(self):
+        return self.locals_scope
+
+
+class StatementSetLocals(StatementSetLocalsMixin, StatementSetLocalsBase):
+    kind = "STATEMENT_SET_LOCALS"
+
+    named_children = ("new_locals",)
+    node_attributes = ("locals_scope",)
+    auto_compute_handling = "operation"
+
+    # TODO: Convert to StatementSetLocals if known to be constant dictionary.
+
+    def getDetailsForDisplay(self):
+        return {"locals_scope": self.locals_scope.getCodeName()}
 
     def getLocalsScope(self):
         return self.locals_scope
@@ -551,25 +600,37 @@ class StatementSetLocals(StatementChildHavingBase):
     def mayRaiseException(self, exception_type):
         return self.subnode_new_locals.mayRaiseException(exception_type)
 
-    def computeStatement(self, trace_collection):
-        new_locals = trace_collection.onExpression(self.subnode_new_locals)
+    def computeStatementOperation(self, trace_collection):
+        self.locals_scope.setTypeShape(self.subnode_new_locals.getTypeShape())
 
-        if new_locals.willRaiseException(BaseException):
-            from .NodeMakingHelpers import (
-                makeStatementExpressionOnlyReplacementNode,
-            )
-
-            result = makeStatementExpressionOnlyReplacementNode(
-                expression=new_locals, node=self
-            )
+        if self.locals_scope.isMarkedForPropagation():
+            self.finalize()
 
             return (
-                result,
-                "new_raise",
+                None,
+                "new_statements",
                 """\
-Setting locals already raises implicitly building new locals.""",
+Forward propagating locals.""",
             )
 
+        if self.subnode_new_locals.mayRaiseException(BaseException):
+            trace_collection.onExceptionRaiseExit(BaseException)
+
+        return self, None, None
+
+    @staticmethod
+    def getStatementNiceName():
+        return "locals mapping init statement"
+
+
+class StatementSetLocalsDictionary(
+    StatementSetLocalsMixin, StatementSetLocalsDictionaryBase
+):
+    kind = "STATEMENT_SET_LOCALS_DICTIONARY"
+
+    node_attributes = ("locals_scope",)
+
+    def computeStatement(self, trace_collection):
         if self.locals_scope.isMarkedForPropagation():
             self.finalize()
 
@@ -582,24 +643,6 @@ Forward propagating locals.""",
 
         return self, None, None
 
-    @staticmethod
-    def getStatementNiceName():
-        return "locals mapping init statement"
-
-
-class StatementSetLocalsDictionary(StatementSetLocals):
-    kind = "STATEMENT_SET_LOCALS_DICTIONARY"
-
-    def __init__(self, locals_scope, source_ref):
-        StatementSetLocals.__init__(
-            self,
-            locals_scope=locals_scope,
-            new_locals=ExpressionConstantDictEmptyRef(
-                source_ref=source_ref, user_provided=True
-            ),
-            source_ref=source_ref,
-        )
-
     @staticmethod
     def mayRaiseException(exception_type):
         return False
@@ -609,6 +652,15 @@ class StatementSetLocalsDictionary(StatementSetLocals):
         return "locals dictionary init statement"
 
 
+class ExpressionLocalsDictRef(ExpressionBuiltinLocalsRef):
+    kind = "EXPRESSION_LOCALS_DICT_REF"
+
+    # Only way we use it.
+    @staticmethod
+    def isFinalUseOfLocals():
+        return True
+
+
 class StatementReleaseLocals(StatementBase):
     kind = "STATEMENT_RELEASE_LOCALS"
 
@@ -649,6 +701,11 @@ class StatementReleaseLocals(StatementBase):
     def getDetails(self):
         return {"locals_scope": self.locals_scope}
 
+    def getDetailsForDisplay(self):
+        return {
+            "locals_scope": self.locals_scope.getCodeName(),
+        }
+
     def getLocalsScope(self):
         return self.locals_scope
 
