@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2023, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -15,30 +15,38 @@
 #     See the License for the specific language governing permissions and
 #     limitations under the License.
 #
-""" The type1 node.
+""" The type nodes.
 
-This one just determines types. It's great for optimization. We may be able to
-predict its value, but knowing it. In that case, we have a built-in name
-reference for that type to convert to, or when checking the result of it, we
-will then know it's limited after the fact.
+These ones deal with types and they are great for optimization. We need to know
+them, their relationship or check for them in re-formulations.
 
 """
 
 from nuitka.Builtins import builtin_names
+from nuitka.Options import isExperimental
 
 from .BuiltinRefNodes import (
     ExpressionBuiltinAnonymousRef,
     ExpressionBuiltinRef,
     makeExpressionBuiltinRef,
 )
-from .ExpressionBases import (
-    ExpressionBuiltinSingleArgBase,
-    ExpressionChildHavingBase,
-    ExpressionChildrenHavingBase,
+from .ChildrenHavingMixins import (
+    ChildHavingClsMixin,
+    ChildrenExpressionBuiltinIssubclassMixin,
+    ChildrenExpressionBuiltinSuper0Mixin,
+    ChildrenExpressionBuiltinSuper1Mixin,
+    ChildrenExpressionBuiltinSuper2Mixin,
+    ChildrenHavingInstanceClassesMixin,
 )
+from .ExpressionBases import ExpressionBase, ExpressionBuiltinSingleArgBase
+from .ExpressionBasesGenerated import ExpressionSubtypeCheckBase
+from .ExpressionShapeMixins import ExpressionBoolShapeExactMixin
 from .NodeBases import SideEffectsFromChildrenMixin
-from .NodeMakingHelpers import wrapExpressionWithNodeSideEffects
-from .shapes.BuiltinTypeShapes import tshape_bool, tshape_type
+from .NodeMakingHelpers import (
+    makeConstantReplacementNode,
+    wrapExpressionWithNodeSideEffects,
+)
+from .shapes.BuiltinTypeShapes import tshape_type
 
 
 class ExpressionBuiltinType1(ExpressionBuiltinSingleArgBase):
@@ -52,21 +60,34 @@ class ExpressionBuiltinType1(ExpressionBuiltinSingleArgBase):
         if type_shape is not None:
             type_name = type_shape.getTypeName()
 
-            if type_name is not None and type_name in __builtins__:
-                result = ExpressionBuiltinRef(
-                    builtin_name=type_name, source_ref=value.getSourceReference()
-                )
+            if type_name is not None:
+                if isExperimental("assume-type-complete") and hasattr(
+                    type_shape, "typical_value"
+                ):
+                    result = makeConstantReplacementNode(
+                        constant=type(getattr(type_shape, "typical_value")),
+                        node=self,
+                        user_provided=False,
+                    )
+                elif type_name in __builtins__:
+                    result = ExpressionBuiltinRef(
+                        builtin_name=type_name, source_ref=value.getSourceReference()
+                    )
+                else:
+                    result = None
 
-                result = wrapExpressionWithNodeSideEffects(
-                    new_node=result, old_node=value
-                )
+                # TODO: Does this even happen to be not None
+                if result is not None:
+                    result = wrapExpressionWithNodeSideEffects(
+                        new_node=result, old_node=value
+                    )
 
-                return (
-                    result,
-                    "new_builtin",
-                    "Replaced predictable type lookup with builtin type '%s'."
-                    % (type_name),
-                )
+                    return (
+                        result,
+                        "new_builtin",
+                        "Replaced predictable type lookup with builtin type '%s'."
+                        % (type_name),
+                    )
 
         if value.isCompileTimeConstant():
             # The above code is supposed to catch these in a better way.
@@ -121,7 +142,29 @@ Removed type taking for unused result.""",
         return self.subnode_value.mayHaveSideEffects()
 
 
-class ExpressionBuiltinSuper2(ExpressionChildrenHavingBase):
+class ExpressionBuiltinSuper1(ChildrenExpressionBuiltinSuper1Mixin, ExpressionBase):
+    """Two arguments form of super."""
+
+    kind = "EXPRESSION_BUILTIN_SUPER1"
+
+    named_children = ("type_arg",)
+
+    def __init__(self, type_arg, source_ref):
+        ChildrenExpressionBuiltinSuper1Mixin.__init__(
+            self,
+            type_arg=type_arg,
+        )
+
+        ExpressionBase.__init__(self, source_ref)
+
+    def computeExpression(self, trace_collection):
+        trace_collection.onExceptionRaiseExit(BaseException)
+
+        # TODO: Quite some cases should be possible to predict.
+        return self, None, None
+
+
+class ExpressionBuiltinSuper2(ChildrenExpressionBuiltinSuper2Mixin, ExpressionBase):
     """Two arguments form of super."""
 
     kind = "EXPRESSION_BUILTIN_SUPER2"
@@ -129,12 +172,14 @@ class ExpressionBuiltinSuper2(ExpressionChildrenHavingBase):
     named_children = ("type_arg", "object_arg")
 
     def __init__(self, type_arg, object_arg, source_ref):
-        ExpressionChildrenHavingBase.__init__(
+        ChildrenExpressionBuiltinSuper2Mixin.__init__(
             self,
-            values={"type_arg": type_arg, "object_arg": object_arg},
-            source_ref=source_ref,
+            type_arg=type_arg,
+            object_arg=object_arg,
         )
 
+        ExpressionBase.__init__(self, source_ref)
+
     def computeExpression(self, trace_collection):
         trace_collection.onExceptionRaiseExit(BaseException)
 
@@ -142,7 +187,7 @@ class ExpressionBuiltinSuper2(ExpressionChildrenHavingBase):
         return self, None, None
 
 
-class ExpressionBuiltinSuper0(ExpressionChildrenHavingBase):
+class ExpressionBuiltinSuper0(ChildrenExpressionBuiltinSuper0Mixin, ExpressionBase):
     """Python3 form of super, arguments determined from cells and function arguments."""
 
     kind = "EXPRESSION_BUILTIN_SUPER0"
@@ -150,12 +195,14 @@ class ExpressionBuiltinSuper0(ExpressionChildrenHavingBase):
     named_children = ("type_arg", "object_arg")
 
     def __init__(self, type_arg, object_arg, source_ref):
-        ExpressionChildrenHavingBase.__init__(
+        ChildrenExpressionBuiltinSuper0Mixin.__init__(
             self,
-            values={"type_arg": type_arg, "object_arg": object_arg},
-            source_ref=source_ref,
+            type_arg=type_arg,
+            object_arg=object_arg,
         )
 
+        ExpressionBase.__init__(self, source_ref)
+
     def computeExpression(self, trace_collection):
         trace_collection.onExceptionRaiseExit(BaseException)
 
@@ -163,18 +210,20 @@ class ExpressionBuiltinSuper0(ExpressionChildrenHavingBase):
         return self, None, None
 
 
-class ExpressionBuiltinIsinstance(ExpressionChildrenHavingBase):
+class ExpressionBuiltinIsinstance(ChildrenHavingInstanceClassesMixin, ExpressionBase):
     kind = "EXPRESSION_BUILTIN_ISINSTANCE"
 
     named_children = ("instance", "classes")
 
     def __init__(self, instance, classes, source_ref):
-        ExpressionChildrenHavingBase.__init__(
+        ChildrenHavingInstanceClassesMixin.__init__(
             self,
-            values={"instance": instance, "classes": classes},
-            source_ref=source_ref,
+            instance=instance,
+            classes=classes,
         )
 
+        ExpressionBase.__init__(self, source_ref)
+
     def computeExpression(self, trace_collection):
         # TODO: Quite some cases should be possible to predict.
 
@@ -204,18 +253,22 @@ class ExpressionBuiltinIsinstance(ExpressionChildrenHavingBase):
         )
 
 
-class ExpressionBuiltinIssubclass(ExpressionChildrenHavingBase):
+class ExpressionBuiltinIssubclass(
+    ChildrenExpressionBuiltinIssubclassMixin, ExpressionBase
+):
     kind = "EXPRESSION_BUILTIN_ISSUBCLASS"
 
     named_children = ("cls", "classes")
 
     def __init__(self, cls, classes, source_ref):
-        ExpressionChildrenHavingBase.__init__(
+        ChildrenExpressionBuiltinIssubclassMixin.__init__(
             self,
-            values={"cls": cls, "classes": classes},
-            source_ref=source_ref,
+            cls=cls,
+            classes=classes,
         )
 
+        ExpressionBase.__init__(self, source_ref)
+
     def computeExpression(self, trace_collection):
         # TODO: Quite some cases should be possible to predict.
 
@@ -245,23 +298,39 @@ class ExpressionBuiltinIssubclass(ExpressionChildrenHavingBase):
         )
 
 
-class ExpressionTypeCheck(SideEffectsFromChildrenMixin, ExpressionChildHavingBase):
+class ExpressionTypeCheck(
+    ExpressionBoolShapeExactMixin,
+    SideEffectsFromChildrenMixin,
+    ChildHavingClsMixin,
+    ExpressionBase,
+):
     kind = "EXPRESSION_TYPE_CHECK"
 
-    named_child = "cls"
+    named_children = ("cls",)
 
     def __init__(self, cls, source_ref):
-        ExpressionChildHavingBase.__init__(
-            self,
-            value=cls,
-            source_ref=source_ref,
-        )
+        ChildHavingClsMixin.__init__(self, cls=cls)
 
-    @staticmethod
-    def getTypeShape():
-        return tshape_bool
+        ExpressionBase.__init__(self, source_ref)
 
     def computeExpression(self, trace_collection):
         # TODO: Quite some cases should be possible to predict, but I am not aware of
         # 100% true Python equivalent at this time.
         return self, None, None
+
+
+class ExpressionSubtypeCheck(
+    ExpressionBoolShapeExactMixin,
+    SideEffectsFromChildrenMixin,
+    ExpressionSubtypeCheckBase,
+):
+    kind = "EXPRESSION_SUBTYPE_CHECK"
+
+    named_children = ("left", "right")
+
+    auto_compute_handling = "final,no_raise"
+
+    def computeExpression(self, trace_collection):
+        # TODO: This needs to check the MRO and can assume the type nature, since it's only coming
+        # from re-formulations that guarantee that.
+        return self, None, None
