@@ -1,4 +1,4 @@
-#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2023, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -276,6 +276,10 @@ class StatementAssignmentVariableMixin(object):
 
         return self, None, None
 
+    def collectVariableAccesses(self, emit_read, emit_write):
+        emit_write(self.variable)
+        self.subnode_source.collectVariableAccesses(emit_read, emit_write)
+
     @abstractmethod
     def hasVeryTrustedValue(self):
         """Does this assignment node have a very trusted value."""
