@@ -1,4 +1,4 @@
-#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2023, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -221,6 +221,9 @@ Check '%s' on dictionary lowered to dictionary '%s'.""" % (
 
         return in_node, tags, message
 
+    def getExpressionDictInConstant(self, value):
+        return self.variable_trace.getDictInValue(value)
+
     def computeExpressionSetSubscript(
         self, set_node, subscript, value_node, trace_collection
     ):
@@ -384,10 +387,6 @@ class ExpressionVariableRef(ExpressionVariableRefBase):
             "owner": self.variable.getOwner().getCodeName(),
         }
 
-    @staticmethod
-    def isExpressionTempVariableRef():
-        return True
-
     @classmethod
     def fromXML(cls, provider, source_ref, **args):
         assert cls is ExpressionVariableRef, cls
@@ -601,6 +600,9 @@ Replaced read-only module attribute '__spec__' with module attribute reference."
 
         return None, None, None
 
+    def collectVariableAccesses(self, emit_read, emit_write):
+        emit_read(self.variable)
+
     def hasShapeListExact(self):
         return (
             self.variable_trace is not None and self.variable_trace.hasShapeListExact()
@@ -623,6 +625,11 @@ Replaced read-only module attribute '__spec__' with module attribute reference."
             and self.variable_trace.hasShapeUnicodeExact()
         )
 
+    def hasShapeBoolExact(self):
+        return (
+            self.variable_trace is not None and self.variable_trace.hasShapeBoolExact()
+        )
+
     def getTruthValue(self):
         return self.variable_trace.getTruthValue()
 
@@ -699,6 +706,10 @@ class ExpressionTempVariableRef(
     def getDetails(self):
         return {"variable": self.variable}
 
+    @staticmethod
+    def isExpressionTempVariableRef():
+        return True
+
     @classmethod
     def fromXML(cls, provider, source_ref, **args):
         assert cls is ExpressionTempVariableRef, cls
