@@ -1,4 +1,4 @@
-#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2023, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -805,7 +805,7 @@ if python_version < 0x300:
                     tried,
                     makeStatementReturn(
                         expression=ExpressionBuiltinExecfile(
-                            in_class_body=node.getParentVariableProvider().isExpressionClassBody(),
+                            in_class_body=node.getParentVariableProvider().isExpressionClassBodyBase(),
                             source_code=makeCallNode(
                                 makeExpressionAttributeLookup(
                                     expression=ExpressionBuiltinOpenP2(
@@ -1032,7 +1032,6 @@ if python_version >= 0x300:
                     tried,
                     makeStatementReturn(
                         expression=ExpressionBuiltinExec(
-                            in_early_closure=node.getParentVariableProvider().isEarlyClosure(),
                             source_code=source,
                             globals_arg=globals_ref,
                             locals_arg=locals_ref,
@@ -1141,7 +1140,7 @@ def super_extractor(node):
             type_arg_owner = class_variable.getOwner()
             if type_arg_owner is provider or not (
                 type_arg_owner.isExpressionFunctionBody()
-                or type_arg_owner.isExpressionClassBody()
+                or type_arg_owner.isExpressionClassBodyBase()
             ):
                 return makeRaiseExceptionReplacementExpression(
                     expression=node,
