@@ -63,6 +63,7 @@ from nuitka.Options import (
 from nuitka.PythonFlavors import isAnacondaPython, isDebianPackagePython
 from nuitka.PythonVersions import getSupportedPythonVersions, python_version
 from nuitka.Tracing import plugins_logger
+from nuitka.utils.Distributions import isDistributionCondaPackage
 from nuitka.utils.Execution import NuitkaCalledProcessError, check_output
 from nuitka.utils.ModuleNames import (
     ModuleName,
@@ -114,6 +115,7 @@ def _getPackageVersion(distribution_name):
             try:
                 from pkg_resources import (
                     DistributionNotFound,
+                    extern,
                     get_distribution,
                 )
             except ImportError:
@@ -125,6 +127,8 @@ def _getPackageVersion(distribution_name):
                     )
                 except DistributionNotFound:
                     result = None
+                except extern.packaging.version.InvalidVersion:
+                    result = None
 
         if result is None:
             # Fallback if nothing is available, which may happen if no package is installed,
@@ -1109,6 +1113,7 @@ except ImportError:
                 "win32": isWin32Windows(),
                 "linux": isLinux(),
                 "anaconda": isAnacondaPython(),
+                "is_conda_package": isDistributionCondaPackage,
                 "debian_python": isDebianPackagePython(),
                 "standalone": isStandaloneMode(),
                 "module_mode": shallMakeModule(),
