@@ -1,4 +1,4 @@
-#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2023, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -43,7 +43,11 @@ from nuitka.freezer.IncludedDataFiles import IncludedDataFile
 from nuitka.freezer.IncludedEntryPoints import IncludedEntryPoint
 from nuitka.ModuleRegistry import addUsedModule
 from nuitka.Tracing import plugins_logger, printLine
-from nuitka.utils.FileOperations import makePath, putTextFileContents
+from nuitka.utils.FileOperations import (
+    getDllBasename,
+    makePath,
+    putTextFileContents,
+)
 from nuitka.utils.Importing import importFileAsModule
 from nuitka.utils.ModuleNames import (
     ModuleName,
@@ -76,12 +80,19 @@ def withPluginProblemReporting(plugin, template, args):
     try:
         yield
     except Exception:  # Catch all the things, pylint: disable=broad-except
+        message = """\
+Plugin issue while working on '%s'. Please report the bug with the above \
+traceback included.""" % (
+            template % args
+        )
+
+        if Options.is_debug:
+            plugin.warning(message)
+            raise
+
         traceback.print_exception(*sys.exc_info())
 
-        plugin.sysexit(
-            "Plugin issue while working on '%s'. Please report the bug with the above traceback included."
-            % (template % args)
-        )
+        plugin.sysexit(message)
 
 
 def withPluginModuleNameProblemReporting(plugin, module_name):
@@ -205,6 +216,14 @@ def getPluginClass(plugin_name):
     plugin_name = Options.getPluginNameConsideringRenames(plugin_name)
 
     if plugin_name not in plugin_name2plugin_classes:
+        for plugin_name2 in plugin_name2plugin_classes:
+            if plugin_name.lower() == plugin_name2.lower():
+                plugins_logger.sysexit(
+                    """\
+Error, unknown plug-in '%s' in wrong case referenced, use '%s' instead."""
+                    % (plugin_name, plugin_name2)
+                )
+
         plugins_logger.sysexit("Error, unknown plug-in '%s' referenced." % plugin_name)
 
     return plugin_name2plugin_classes[plugin_name][0]
@@ -442,7 +461,7 @@ class Plugins(object):
 
     @classmethod
     def getPackageExtraScanPaths(cls, package_name, package_dir):
-        key = package_name
+        key = package_name, package_dir
 
         if key not in cls.extra_scan_paths_cache:
             cls.extra_scan_paths_cache[key] = ()
@@ -745,9 +764,10 @@ class Plugins(object):
                 is_fake=module_name,
                 hide_syntax_error=False,
             )
-        except SyntaxError:
+        except SyntaxError as e:
             plugins_logger.sysexit(
-                "SyntaxError in plugin provided source code for '%s'." % module_name
+                "SyntaxError in plugin provided source code for '%s': %s."
+                % (module_name, e)
             )
 
         if trigger_module.getCompilationMode() == "bytecode":
@@ -758,7 +778,7 @@ class Plugins(object):
     @classmethod
     def onModuleDiscovered(cls, module):
         # We offer plugins many ways to provide extra stuff
-        # pylint: disable=too-many-branches,too-many-locals,too-many-statements
+        # pylint: disable=too-many-locals,too-many-statements
 
         full_name = module.getFullName()
 
@@ -816,50 +836,56 @@ class Plugins(object):
                 _untangleFakeDesc(description=plugin.createFakeModuleDependency(module))
             )
 
-        if pre_module_load_descriptions:
-            total_code = []
+        def combineLoadCodes(module_load_descriptions, mode):
+            future_imports_code = []
+            normal_code_code = []
             total_flags = OrderedSet()
             reasons = []
 
-            for plugin, pre_code, reason, flags in pre_module_load_descriptions:
-                if pre_code:
+            for plugin, code, reason, flags in module_load_descriptions:
+                if code:
                     plugin.info(
-                        "Injecting pre-module load code for module '%s':" % full_name
+                        "Injecting %s-module load code for module '%s':"
+                        % (mode, full_name)
                     )
                     for line in reason.split("\n"):
                         plugin.info("    " + line)
 
-                    total_code.append(pre_code)
+                    for line in code.splitlines():
+                        line = line + "\n"
+
+                        if line.startswith("from __future__"):
+                            future_imports_code.append(line)
+                        else:
+                            normal_code_code.append(line)
+
                     total_flags.update(flags)
                     reasons.append(reason)
 
+            total_code = future_imports_code + normal_code_code
+
+            return total_code, reasons, total_flags
+
+        if pre_module_load_descriptions:
+            total_code, reasons, total_flags = combineLoadCodes(
+                module_load_descriptions=pre_module_load_descriptions, mode="pre"
+            )
+
             if total_code:
                 assert full_name not in pre_modules
 
                 pre_modules[full_name] = cls._createTriggerLoadedModule(
                     module=module,
                     trigger_name=pre_module_load_trigger_name,
-                    code="\n\n".join(total_code),
+                    code="".join(total_code),
                     flags=total_flags,
                 )
                 pre_modules_reasons[full_name] = tuple(reasons)
 
         if post_module_load_descriptions:
-            total_code = []
-            total_flags = OrderedSet()
-            reasons = []
-
-            for plugin, post_code, reason, flags in post_module_load_descriptions:
-                if post_code:
-                    plugin.info(
-                        "Injecting post-module load code for module '%s':" % full_name
-                    )
-                    for line in reason.split("\n"):
-                        plugin.info("    " + line)
-
-                    total_code.append(post_code)
-                    total_flags.update(flags)
-                    reasons.append(reason)
+            total_code, reasons, total_flags = combineLoadCodes(
+                module_load_descriptions=post_module_load_descriptions, mode="post"
+            )
 
             if total_code:
                 assert full_name not in post_modules
@@ -867,7 +893,7 @@ class Plugins(object):
                 post_modules[full_name] = cls._createTriggerLoadedModule(
                     module=module,
                     trigger_name=post_module_load_trigger_name,
-                    code="\n\n".join(total_code),
+                    code="".join(total_code),
                     flags=total_flags,
                 )
                 post_modules_reasons[full_name] = reasons
@@ -908,12 +934,13 @@ class Plugins(object):
         contributing_plugins = OrderedSet()
 
         for plugin in getActivePlugins():
-            new_source_code = plugin.onModuleSourceCode(module_name, source_code)
-            if new_source_code is not None and new_source_code != source_code:
-                source_code = new_source_code
-                contributing_plugins.add(plugin)
+            with withPluginModuleNameProblemReporting(plugin, module_name):
+                new_source_code = plugin.onModuleSourceCode(module_name, source_code)
+                if new_source_code is not None and new_source_code != source_code:
+                    source_code = new_source_code
+                    contributing_plugins.add(plugin)
 
-            assert type(source_code) is str
+                assert type(source_code) is str
 
         return source_code, contributing_plugins
 
@@ -962,7 +989,6 @@ class Plugins(object):
                 )
 
             if result is not None:
-                # false alarm, pylint: disable=unsubscriptable-object
                 if result[0] != must_recurse[0]:
                     plugin.sysexit(
                         "Error, decision %s does not match other plugin '%s' decision."
@@ -1267,6 +1293,89 @@ class Plugins(object):
             for value in plugin.getCacheContributionValues(module_name):
                 yield value
 
+    @classmethod
+    def decideAllowOutsideDependencies(cls, module_name):
+        if not Options.isExperimental("no-outside-dependencies"):
+            return None
+
+        result = None
+        plugin_name = None
+
+        if module_name is not None:
+            for plugin in getActivePlugins():
+                value = plugin.decideAllowOutsideDependencies(module_name)
+
+                if value is True:
+                    if result is False:
+                        plugin.sysexit(
+                            "Error, conflicting allow/disallow outside dependencies of plug-in '%s'."
+                            % plugin_name
+                        )
+
+                    result = True
+                    plugin_name = plugin.plugin_name
+
+                elif value is False:
+                    if result is False:
+                        plugin.sysexit(
+                            "Error, conflicting allow/disallow outside dependencies of plug-in '%s'."
+                            % plugin_name
+                        )
+
+                    result = False
+                    plugin_name = plugin.plugin_name
+                elif value is not None:
+                    plugin.sysexit(
+                        "Error, can only return True, False, None from 'decideAllowOutsideDependencies' not %r"
+                        % value
+                    )
+
+        return result
+
+    @classmethod
+    def isAcceptableMissingDLL(cls, package_name, filename):
+        dll_basename = getDllBasename(os.path.basename(filename))
+
+        # Not a DLL filename, then it cannot be true, but it's kind of strange
+        # to get asked.
+        if dll_basename is None:
+            return False
+
+        result = None
+        plugin_name = None
+
+        for plugin in getActivePlugins():
+            value = plugin.isAcceptableMissingDLL(
+                package_name=package_name, dll_basename=dll_basename
+            )
+
+            if value is True:
+                if result is False:
+                    plugin.sysexit(
+                        "Error, conflicting accept/reject missing DLLs of plug-in '%s'."
+                        % plugin_name
+                    )
+
+                result = True
+                plugin_name = plugin.plugin_name
+
+            elif value is False:
+                if result is False:
+                    plugin.sysexit(
+                        "Error, conflicting accept/reject missing DLLs of plug-in '%s'."
+                        % plugin_name
+                    )
+
+                result = False
+                plugin_name = plugin.plugin_name
+            elif value is not None:
+                plugin.sysexit(
+                    "Error, can only return True, False, None from 'isAcceptableMissingDLL' not %r"
+                    % value
+                )
+
+        return result, plugin_name
+
 
 def listPlugins():
     """Print available standard plugins."""
@@ -1277,16 +1386,22 @@ def listPlugins():
     printLine("-" * 80)
 
     plist = []
-    name_len = 0
+    max_name_length = 0
     for plugin_name in sorted(plugin_name2plugin_classes):
         plugin = plugin_name2plugin_classes[plugin_name][0]
+
+        if plugin.isDeprecated():
+            continue
+
         if hasattr(plugin, "plugin_desc"):
             plist.append((plugin_name, plugin.plugin_desc))
         else:
             plist.append((plugin_name, ""))
-        name_len = max(len(plugin_name) + 1, name_len)
+
+        max_name_length = max(len(plugin_name), max_name_length)
+
     for line in plist:
-        printLine(" " + line[0].ljust(name_len), line[1])
+        printLine(" " + line[0].ljust(max_name_length + 1), line[1])
 
 
 def isObjectAUserPluginBaseClass(obj):
