@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2023, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -24,7 +24,7 @@ from nuitka.plugins.PluginBase import NuitkaPluginBase, standalone_only
 
 class NuitkaPluginGi(NuitkaPluginBase):
     plugin_name = "gi"
-    plugin_desc = "Support for GI typelib dependency"
+    plugin_desc = "Support for GI package typelib dependency."
 
     @staticmethod
     def isAlwaysEnabled():
@@ -40,7 +40,7 @@ class NuitkaPluginGi(NuitkaPluginBase):
         if module.getFullName() == "gi":
             code = r"""
 import os
-if not os.environ.get("GI_TYPELIB_PATH="):
+if not os.environ.get("GI_TYPELIB_PATH"):
     os.environ["GI_TYPELIB_PATH"] = os.path.join(__nuitka_binary_dir, "girepository")"""
 
             return code, "Set typelib search path"
@@ -49,7 +49,7 @@ if not os.environ.get("GI_TYPELIB_PATH="):
     def considerDataFiles(self, module):
         """Copy typelib files from the default installation path"""
         if module.getFullName() == "gi":
-            path = self.queryRuntimeInformationMultiple(
+            gi_typelib_info = self.queryRuntimeInformationMultiple(
                 info_name="gi_info",
                 setup_codes="import gi; from gi.repository import GObject",
                 values=(
@@ -60,12 +60,15 @@ if not os.environ.get("GI_TYPELIB_PATH="):
                 ),
             )
 
-            gi_repository_path = os.path.dirname(path.introspection_module)
-            yield self.makeIncludedDataDirectory(
-                source_path=gi_repository_path,
-                dest_path="girepository",
-                reason="typelib files for gi modules",
-            )
+            if gi_typelib_info is not None:
+                gi_repository_path = os.path.dirname(
+                    gi_typelib_info.introspection_module
+                )
+                yield self.makeIncludedDataDirectory(
+                    source_path=gi_repository_path,
+                    dest_path="girepository",
+                    reason="typelib files for gi modules",
+                )
 
     @staticmethod
     def getImplicitImports(module):
@@ -91,5 +94,8 @@ if not os.environ.get("GI_TYPELIB_PATH="):
 
             if gtk_dll_path is not None:
                 yield self.makeDllEntryPoint(
-                    gtk_dll_path, os.path.basename(gtk_dll_path), None
+                    source_path=gtk_dll_path,
+                    dest_path=os.path.basename(gtk_dll_path),
+                    package_name=None,
+                    reason="needed by 'gi._gi'",
                 )
