@@ -1,4 +1,4 @@
-#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2023, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -41,7 +41,7 @@ class NuitkaPluginGlfw(NuitkaPluginBase):
 
     # TODO: Maybe rename to opengl maybe
     plugin_name = "glfw"  # Nuitka knows us by this name
-    plugin_desc = "Required for OpenGL and glfw in standalone mode"
+    plugin_desc = "Required for OpenGL and 'glfw' package in standalone mode."
 
     @staticmethod
     def isAlwaysEnabled():
@@ -114,6 +114,7 @@ class NuitkaPluginGlfw(NuitkaPluginBase):
                 source_path=dll_filename,
                 dest_path=os.path.join("glfw", os.path.basename(dll_filename)),
                 package_name="glfw.library",
+                reason="needed by 'glfw'",
             )
 
     def createPreModuleLoadCode(self, module):
@@ -122,7 +123,7 @@ class NuitkaPluginGlfw(NuitkaPluginBase):
 
             code = r"""
 import os
-os.environ["PYGLFW_LIBRARY"] = os.path.join(__nuitka_binary_dir, "glfw", %r)
+os.environ["PYGLFW_LIBRARY"] = os.path.join(__nuitka_binary_dir, "glfw", "%s")
 """ % os.path.basename(
                 dll_filename
             )
