@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2023, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -31,7 +31,7 @@ from nuitka.ModuleRegistry import (
 )
 from nuitka.plugins.PluginBase import NuitkaPluginBase
 from nuitka.PythonVersions import python_version
-from nuitka.tree.SourceReading import readSourceCodeFromFilename
+from nuitka.tree.SourceHandling import readSourceCodeFromFilename
 from nuitka.utils.ModuleNames import ModuleName
 
 
@@ -49,7 +49,7 @@ class NuitkaPluginMultiprocessingWorkarounds(NuitkaPluginBase):
     """
 
     plugin_name = "multiprocessing"
-    plugin_desc = "Required by Python's multiprocessing module"
+    plugin_desc = "Required by Python's 'multiprocessing' module."
 
     @classmethod
     def isRelevant(cls):
@@ -72,11 +72,15 @@ class NuitkaPluginMultiprocessingWorkarounds(NuitkaPluginBase):
             code = """\
 import sys, os
 sys.frozen = 1
-if sys.platform == "win32" and not os.path.exists(sys.argv[0]) and not sys.argv[0].endswith(".exe"):
-    sys.executable = sys.argv[0] + ".exe"
-else:
-    sys.executable = sys.argv[0]
-"""
+argv0 = sys.argv[0]
+if sys.platform == "win32" and not os.path.exists(argv0) and not argv0.endswith(".exe"):
+    argv0 += ".exe"
+
+sys.executable = %s
+sys._base_executable = sys.executable
+""" % (
+                "__nuitka_binary_exe" if Options.isStandaloneMode() else "argv0"
+            )
             return (
                 code,
                 """\
