@@ -1,4 +1,4 @@
-#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2023, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -42,11 +42,11 @@ from nuitka.nodes.ConstantRefNodes import (
     makeConstantRefNode,
 )
 from nuitka.nodes.ContainerMakingNodes import makeExpressionMakeTupleOrConstant
-from nuitka.nodes.ContainerOperationNodes import ExpressionListOperationPop
 from nuitka.nodes.InjectCNodes import (
     StatementInjectCCode,
     StatementInjectCDecl,
 )
+from nuitka.nodes.ListOperationNodes import ExpressionListOperationPop1
 from nuitka.nodes.NodeMakingHelpers import (
     makeRaiseExceptionExpressionFromTemplate,
 )
@@ -76,7 +76,10 @@ from nuitka.nodes.VariableNameNodes import (
     StatementDelVariableName,
 )
 from nuitka.nodes.VariableRefNodes import ExpressionTempVariableRef
-from nuitka.nodes.VariableReleaseNodes import makeStatementReleaseVariable
+from nuitka.nodes.VariableReleaseNodes import (
+    makeStatementReleaseVariable,
+    makeStatementsReleaseVariables,
+)
 from nuitka.Options import hasPythonFlagNoAnnotations, isExperimental
 from nuitka.PythonVersions import python_version
 from nuitka.Tracing import general
@@ -122,7 +125,7 @@ def buildExtSliceNode(provider, node, source_ref):
         elements.append(element)
 
     return makeExpressionMakeTupleOrConstant(
-        elements=elements, user_provided=True, source_ref=source_ref
+        elements=tuple(elements), user_provided=True, source_ref=source_ref
     )
 
 
@@ -180,9 +183,9 @@ def buildAssignmentStatementsFromDecoded(provider, kind, detail, source, source_
         # For Python3 there is no slicing operation, this is always done
         # with subscript using a slice object. For Python2, it is only done
         # if no "step" is provided.
-        use_sliceobj = python_version >= 0x300
+        use_slice_object = python_version >= 0x300
 
-        if use_sliceobj:
+        if use_slice_object:
             return StatementAssignmentSubscript(
                 subscribed=lookup_source,
                 source=source,
@@ -239,7 +242,7 @@ def buildAssignmentStatementsFromDecoded(provider, kind, detail, source, source_
                     starred_index + 1,
                     makeStatementAssignmentVariable(
                         variable=element_var,
-                        source=ExpressionListOperationPop(
+                        source=ExpressionListOperationPop1(
                             list_arg=ExpressionTempVariableRef(
                                 variable=starred_list_var, source_ref=source_ref
                             ),
@@ -594,7 +597,7 @@ def buildAnnAssignNode(provider, node, source_ref):
     """Python3.6 annotation assignment."""
     # There are many cases to deal with here.
 
-    if provider.isCompiledPythonModule() or provider.isExpressionClassBody():
+    if provider.isCompiledPythonModule() or provider.isExpressionClassBodyBase():
         provider.markAsNeedsAnnotationsDictionary()
 
     # Evaluate the right hand side first, so it can get names provided
@@ -633,7 +636,7 @@ def buildAnnAssignNode(provider, node, source_ref):
     # they are ignored like comments.
     if variable_name is not None:
         if not hasPythonFlagNoAnnotations() and (
-            provider.isExpressionClassBody() or provider.isCompiledPythonModule()
+            provider.isExpressionClassBodyBase() or provider.isCompiledPythonModule()
         ):
             annotation = buildAnnotationNode(provider, node.annotation, source_ref)
 
@@ -700,9 +703,9 @@ def buildDeleteStatementFromDecoded(provider, kind, detail, source_ref):
     elif kind == "Slice":
         lookup_source, lower, upper = detail
 
-        use_sliceobj = python_version >= 0x300
+        use_slice_object = python_version >= 0x300
 
-        if use_sliceobj:
+        if use_slice_object:
             return StatementDelSubscript(
                 subscribed=lookup_source,
                 subscript=makeExpressionBuiltinSlice(
@@ -908,16 +911,9 @@ def _buildInplaceAssignSubscriptNode(
         makeTryFinallyStatement(
             provider=provider,
             tried=statements,
-            final=(
-                makeStatementReleaseVariable(
-                    variable=tmp_variable1, source_ref=source_ref
-                ),
-                makeStatementReleaseVariable(
-                    variable=tmp_variable2, source_ref=source_ref
-                ),
-                makeStatementReleaseVariable(
-                    variable=tmp_variable3, source_ref=source_ref
-                ),
+            final=makeStatementsReleaseVariables(
+                variables=(tmp_variable1, tmp_variable2, tmp_variable3),
+                source_ref=source_ref,
             ),
             source_ref=source_ref,
         ),
@@ -994,10 +990,10 @@ def _buildInplaceAssignSliceNode(
 
         upper_ref1 = upper_ref2 = None
 
-    use_sliceobj = python_version >= 0x300
+    use_slice_object = python_version >= 0x300
 
     # Second assign the in-place result over the original value.
-    if use_sliceobj:
+    if use_slice_object:
         statements += (
             makeStatementAssignmentVariable(
                 variable=tmp_variable4,
@@ -1244,8 +1240,7 @@ def buildNamedExprNode(provider, node, source_ref):
         ),
     )
 
-    outline_body.setChild(
-        "body",
+    outline_body.setChildBody(
         makeStatementsSequenceFromStatement(
             statement=makeTryFinallyStatement(
                 provider=provider,
@@ -1255,7 +1250,7 @@ def buildNamedExprNode(provider, node, source_ref):
                 ),
                 source_ref=source_ref,
             )
-        ),
+        )
     )
 
     return outline_body
