@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2023, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -17,20 +17,15 @@
 #
 """ Reformulation of Python3 class statements.
 
-Consult the developer manual for information. TODO: Add ability to sync
-source code comments with developer manual sections.
+Consult the Developer Manual for information. TODO: Add ability to sync
+source code comments with Developer Manual sections.
 
 """
 
-from nuitka.nodes.AssignNodes import (
-    StatementAssignmentVariable,
-    StatementAssignmentVariableName,
-    StatementReleaseVariable,
-)
 from nuitka.nodes.AttributeNodes import (
     ExpressionAttributeCheck,
-    ExpressionAttributeLookup,
     ExpressionBuiltinGetattr,
+    makeExpressionAttributeLookup,
 )
 from nuitka.nodes.BuiltinIteratorNodes import ExpressionBuiltinIter1
 from nuitka.nodes.BuiltinNextNodes import ExpressionBuiltinNext1
@@ -38,7 +33,7 @@ from nuitka.nodes.BuiltinRefNodes import makeExpressionBuiltinTypeRef
 from nuitka.nodes.BuiltinTypeNodes import ExpressionBuiltinTuple
 from nuitka.nodes.CallNodes import makeExpressionCall
 from nuitka.nodes.ClassNodes import (
-    ExpressionClassBody,
+    ExpressionClassBodyP3,
     ExpressionSelectMetaclass,
 )
 from nuitka.nodes.CodeObjectSpecs import CodeObjectSpec
@@ -52,24 +47,26 @@ from nuitka.nodes.ContainerMakingNodes import (
     makeExpressionMakeTuple,
     makeExpressionMakeTupleOrConstant,
 )
-from nuitka.nodes.ContainerOperationNodes import (
-    ExpressionListOperationExtend,
-    StatementListOperationAppend,
-)
+from nuitka.nodes.ContainerOperationNodes import StatementListOperationAppend
 from nuitka.nodes.DictionaryNodes import (
-    ExpressionDictOperationGet,
+    ExpressionDictOperationGet2,
     ExpressionDictOperationIn,
     StatementDictOperationRemove,
     StatementDictOperationUpdate,
 )
+from nuitka.nodes.ExceptionNodes import (
+    ExpressionBuiltinMakeException,
+    StatementRaiseException,
+)
 from nuitka.nodes.FunctionAttributeNodes import ExpressionFunctionQualnameRef
 from nuitka.nodes.FunctionNodes import (
-    ExpressionFunctionCall,
-    ExpressionFunctionCreation,
     ExpressionFunctionRef,
+    makeExpressionFunctionCall,
+    makeExpressionFunctionCreation,
 )
-from nuitka.nodes.GlobalsLocalsNodes import ExpressionBuiltinLocalsRef
+from nuitka.nodes.ListOperationNodes import ExpressionListOperationExtend
 from nuitka.nodes.LocalsDictNodes import (
+    ExpressionLocalsDictRef,
     StatementLocalsDictOperationSet,
     StatementReleaseLocals,
     StatementSetLocals,
@@ -82,12 +79,20 @@ from nuitka.nodes.NodeMakingHelpers import (
 )
 from nuitka.nodes.ReturnNodes import StatementReturn
 from nuitka.nodes.StatementNodes import StatementExpressionOnly
-from nuitka.nodes.SubscriptNodes import ExpressionSubscriptLookup
-from nuitka.nodes.TypeNodes import ExpressionBuiltinType1, ExpressionTypeCheck
+from nuitka.nodes.SubscriptNodes import makeExpressionIndexLookup
+from nuitka.nodes.TypeNodes import (
+    ExpressionBuiltinType1,
+    ExpressionSubtypeCheck,
+    ExpressionTypeCheck,
+)
+from nuitka.nodes.VariableAssignNodes import makeStatementAssignmentVariable
+from nuitka.nodes.VariableNameNodes import StatementAssignmentVariableName
 from nuitka.nodes.VariableRefNodes import (
     ExpressionTempVariableRef,
     ExpressionVariableRef,
 )
+from nuitka.nodes.VariableReleaseNodes import makeStatementsReleaseVariables
+from nuitka.Options import isExperimental
 from nuitka.PythonVersions import python_version
 from nuitka.specs.ParameterSpecs import ParameterSpec
 
@@ -102,7 +107,7 @@ from .ReformulationTryFinallyStatements import makeTryFinallyStatement
 from .TreeHelpers import (
     buildFrameNode,
     buildNode,
-    buildNodeList,
+    buildNodeTuple,
     extractDocFromBody,
     getKind,
     makeDictCreationOrConstant2,
@@ -122,7 +127,7 @@ def _buildBasesTupleCreationNode(provider, elements, source_ref):
             )
 
     return makeExpressionMakeTupleOrConstant(
-        elements=buildNodeList(provider, elements, source_ref),
+        elements=buildNodeTuple(provider, elements, source_ref),
         user_provided=True,
         source_ref=source_ref,
     )
@@ -134,7 +139,7 @@ def buildClassNode3(provider, node, source_ref):
     # pylint: disable=I0021,too-many-branches,too-many-locals,too-many-statements
 
     # This function is the Python3 special case with special re-formulation as
-    # according to developer manual.
+    # according to Developer Manual.
     class_statement_nodes, class_doc = extractDocFromBody(node)
 
     # We need a scope for the temporary variables, and they might be closured.
@@ -148,7 +153,7 @@ def buildClassNode3(provider, node, source_ref):
     )
     tmp_prepared = provider.allocateTempVariable(temp_scope=temp_scope, name="prepared")
 
-    class_creation_function = ExpressionClassBody(
+    class_creation_function = ExpressionClassBodyP3(
         provider=provider, name=node.name, doc=class_doc, source_ref=source_ref
     )
 
@@ -158,6 +163,7 @@ def buildClassNode3(provider, node, source_ref):
     class_variable = class_locals_scope.getLocalVariable(
         owner=class_creation_function, variable_name="__class__"
     )
+
     class_locals_scope.registerProvidedVariable(class_variable)
 
     class_variable_ref = ExpressionVariableRef(
@@ -168,6 +174,7 @@ def buildClassNode3(provider, node, source_ref):
 
     code_object = CodeObjectSpec(
         co_name=node.name,
+        co_qualname=provider.getChildQualname(node.name),
         co_kind="Class",
         co_varnames=(),
         co_freevars=(),
@@ -243,7 +250,7 @@ def buildClassNode3(provider, node, source_ref):
         StatementLocalsDictOperationSet(
             locals_scope=locals_scope,
             variable_name="__qualname__",
-            value=qualname_ref,
+            source=qualname_ref,
             source_ref=source_ref,
         )
     )
@@ -256,7 +263,7 @@ def buildClassNode3(provider, node, source_ref):
             StatementLocalsDictOperationSet(
                 locals_scope=locals_scope,
                 variable_name="__annotations__",
-                value=makeConstantRefNode(
+                source=makeConstantRefNode(
                     constant={}, source_ref=source_ref, user_provided=True
                 ),
                 source_ref=source_ref,
@@ -297,7 +304,7 @@ def buildClassNode3(provider, node, source_ref):
                 yes_branch=StatementLocalsDictOperationSet(
                     locals_scope=locals_scope,
                     variable_name="__orig_bases__",
-                    value=ExpressionTempVariableRef(
+                    source=ExpressionTempVariableRef(
                         variable=tmp_bases_orig, source_ref=source_ref
                     ),
                     source_ref=source_ref,
@@ -308,7 +315,7 @@ def buildClassNode3(provider, node, source_ref):
         )
 
     statements += (
-        StatementAssignmentVariable(
+        makeStatementAssignmentVariable(
             variable=class_variable,
             source=makeExpressionCall(
                 called=ExpressionTempVariableRef(
@@ -322,7 +329,7 @@ def buildClassNode3(provider, node, source_ref):
                             user_provided=True,
                         ),
                         makeBasesRef(),
-                        ExpressionBuiltinLocalsRef(
+                        ExpressionLocalsDictRef(
                             locals_scope=locals_scope, source_ref=source_ref
                         ),
                     ),
@@ -351,7 +358,7 @@ def buildClassNode3(provider, node, source_ref):
 
     # The class body is basically a function that implicitly, at the end
     # returns its locals and cannot have other return statements contained.
-    class_creation_function.setChild("body", body)
+    class_creation_function.setChildBody(body)
 
     # The class body is basically a function that implicitly, at the end
     # returns its created class and cannot have other return statements
@@ -359,7 +366,9 @@ def buildClassNode3(provider, node, source_ref):
 
     decorated_body = class_creation_function
 
-    for decorator in buildNodeList(provider, reversed(node.decorator_list), source_ref):
+    for decorator in buildNodeTuple(
+        provider, reversed(node.decorator_list), source_ref
+    ):
         decorated_body = makeExpressionCall(
             called=decorator,
             args=makeExpressionMakeTuple(
@@ -378,7 +387,7 @@ def buildClassNode3(provider, node, source_ref):
 
     if node.bases:
         statements.append(
-            StatementAssignmentVariable(
+            makeStatementAssignmentVariable(
                 variable=tmp_bases if python_version < 0x370 else tmp_bases_orig,
                 source=_buildBasesTupleCreationNode(
                     provider=provider, elements=node.bases, source_ref=source_ref
@@ -388,8 +397,8 @@ def buildClassNode3(provider, node, source_ref):
         )
 
         if python_version >= 0x370:
-            bases_conversion = ExpressionFunctionCall(
-                function=ExpressionFunctionCreation(
+            bases_conversion = makeExpressionFunctionCall(
+                function=makeExpressionFunctionCreation(
                     function_ref=ExpressionFunctionRef(
                         function_body=getClassBasesMroConversionHelper(),
                         source_ref=source_ref,
@@ -408,13 +417,13 @@ def buildClassNode3(provider, node, source_ref):
             )
 
             statements.append(
-                StatementAssignmentVariable(
+                makeStatementAssignmentVariable(
                     variable=tmp_bases, source=bases_conversion, source_ref=source_ref
                 )
             )
 
     statements.append(
-        StatementAssignmentVariable(
+        makeStatementAssignmentVariable(
             variable=tmp_class_decl_dict,
             source=makeDictCreationOrConstant2(
                 keys=[keyword.arg for keyword in keywords],
@@ -444,13 +453,11 @@ def buildClassNode3(provider, node, source_ref):
     # decl dict of course.
     if node.bases:
         unspecified_metaclass_expression = ExpressionBuiltinType1(
-            value=ExpressionSubscriptLookup(
+            value=makeExpressionIndexLookup(
                 expression=ExpressionTempVariableRef(
                     variable=tmp_bases, source_ref=source_ref
                 ),
-                subscript=makeConstantRefNode(
-                    constant=0, source_ref=source_ref, user_provided=True
-                ),
+                index_value=0,
                 source_ref=source_ref,
             ),
             source_ref=source_ref,
@@ -474,10 +481,10 @@ def buildClassNode3(provider, node, source_ref):
             builtin_name="type", source_ref=source_ref
         )
 
-    call_prepare = StatementAssignmentVariable(
+    call_prepare = makeStatementAssignmentVariable(
         variable=tmp_prepared,
         source=makeExpressionCall(
-            called=ExpressionAttributeLookup(
+            called=makeExpressionAttributeLookup(
                 expression=ExpressionTempVariableRef(
                     variable=tmp_metaclass, source_ref=source_ref
                 ),
@@ -529,7 +536,7 @@ def buildClassNode3(provider, node, source_ref):
                             ),
                             source_ref=source_ref,
                         ),
-                        ExpressionAttributeLookup(
+                        makeExpressionAttributeLookup(
                             expression=ExpressionBuiltinType1(
                                 value=ExpressionTempVariableRef(
                                     variable=tmp_prepared, source_ref=source_ref
@@ -547,9 +554,9 @@ def buildClassNode3(provider, node, source_ref):
         )
 
     statements += (
-        StatementAssignmentVariable(
+        makeStatementAssignmentVariable(
             variable=tmp_metaclass,
-            source=ExpressionSelectMetaclass(
+            source=makeExpressionSelectMetaclass(
                 metaclass=ExpressionConditional(
                     condition=ExpressionDictOperationIn(
                         key=makeConstantRefNode(
@@ -562,7 +569,7 @@ def buildClassNode3(provider, node, source_ref):
                         ),
                         source_ref=source_ref,
                     ),
-                    expression_yes=ExpressionDictOperationGet(
+                    expression_yes=ExpressionDictOperationGet2(
                         dict_arg=ExpressionTempVariableRef(
                             variable=tmp_class_decl_dict, source_ref=source_ref
                         ),
@@ -612,7 +619,7 @@ def buildClassNode3(provider, node, source_ref):
                 source_ref=source_ref,
             ),
             yes_branch=call_prepare,
-            no_branch=StatementAssignmentVariable(
+            no_branch=makeStatementAssignmentVariable(
                 variable=tmp_prepared,
                 source=makeConstantRefNode(
                     constant={}, source_ref=source_ref, user_provided=True
@@ -632,24 +639,23 @@ def buildClassNode3(provider, node, source_ref):
     if python_version >= 0x340:
         class_creation_function.qualname_setup = node.name, qualname_assign
 
-    final = [tmp_class_decl_dict, tmp_metaclass, tmp_prepared]
+    tmp_variables = [tmp_class_decl_dict, tmp_metaclass, tmp_prepared]
     if node.bases:
-        final.insert(0, tmp_bases)
+        tmp_variables.insert(0, tmp_bases)
         if python_version >= 0x370:
-            final.insert(0, tmp_bases_orig)
+            tmp_variables.insert(0, tmp_bases_orig)
 
     return makeTryFinallyStatement(
         provider=provider,
         tried=statements,
-        final=tuple(
-            StatementReleaseVariable(variable=variable, source_ref=source_ref)
-            for variable in final
+        final=makeStatementsReleaseVariables(
+            variables=tmp_variables, source_ref=source_ref
         ),
         source_ref=source_ref,
     )
 
 
-# Note: This emulates Python/bltinmodule.c/update_bases function. We have it
+# Note: This emulates "Python/bltinmodule.c/update_bases" function. We have it
 # here, so we can hope to statically optimize it later on.
 @once_decorator
 def getClassBasesMroConversionHelper():
@@ -691,7 +697,7 @@ def getClassBasesMroConversionHelper():
                     variable=tmp_result_variable, source_ref=internal_source_ref
                 ),
                 value=makeExpressionCall(
-                    called=ExpressionAttributeLookup(
+                    called=makeExpressionAttributeLookup(
                         expression=ExpressionTempVariableRef(
                             variable=tmp_item_variable, source_ref=internal_source_ref
                         ),
@@ -737,7 +743,7 @@ def getClassBasesMroConversionHelper():
 
     loop_body = makeStatementsSequenceFromStatements(
         makeTryExceptSingleHandlerNode(
-            tried=StatementAssignmentVariable(
+            tried=makeStatementAssignmentVariable(
                 variable=tmp_item_variable,
                 source=ExpressionBuiltinNext1(
                     value=ExpressionTempVariableRef(
@@ -764,23 +770,8 @@ def getClassBasesMroConversionHelper():
         ),
     )
 
-    final = (
-        StatementReleaseVariable(
-            variable=args_variable, source_ref=internal_source_ref
-        ),
-        StatementReleaseVariable(
-            variable=tmp_result_variable, source_ref=internal_source_ref
-        ),
-        StatementReleaseVariable(
-            variable=tmp_iter_variable, source_ref=internal_source_ref
-        ),
-        StatementReleaseVariable(
-            variable=tmp_item_variable, source_ref=internal_source_ref
-        ),
-    )
-
     tried = makeStatementsSequenceFromStatements(
-        StatementAssignmentVariable(
+        makeStatementAssignmentVariable(
             variable=tmp_iter_variable,
             source=ExpressionBuiltinIter1(
                 value=ExpressionVariableRef(
@@ -790,7 +781,7 @@ def getClassBasesMroConversionHelper():
             ),
             source_ref=internal_source_ref,
         ),
-        StatementAssignmentVariable(
+        makeStatementAssignmentVariable(
             variable=tmp_result_variable,
             source=makeConstantRefNode(constant=[], source_ref=internal_source_ref),
             source_ref=internal_source_ref,
@@ -807,16 +798,230 @@ def getClassBasesMroConversionHelper():
         ),
     )
 
-    result.setChild(
-        "body",
+    result.setChildBody(
         makeStatementsSequenceFromStatement(
             makeTryFinallyStatement(
                 provider=result,
                 tried=tried,
-                final=final,
+                final=makeStatementsReleaseVariables(
+                    variables=(
+                        args_variable,
+                        tmp_result_variable,
+                        tmp_iter_variable,
+                        tmp_item_variable,
+                    ),
+                    source_ref=internal_source_ref,
+                ),
                 source_ref=internal_source_ref,
             )
+        )
+    )
+
+    return result
+
+
+def makeExpressionSelectMetaclass(metaclass, bases, source_ref):
+    if isExperimental("select-metaclass-helper"):
+        return makeExpressionFunctionCall(
+            function=makeExpressionFunctionCreation(
+                function_ref=ExpressionFunctionRef(
+                    function_body=getClassSelectMetaClassHelper(),
+                    source_ref=source_ref,
+                ),
+                defaults=(),
+                kw_defaults=None,
+                annotations=None,
+                source_ref=source_ref,
+            ),
+            values=(metaclass, bases),
+            source_ref=source_ref,
+        )
+
+    else:
+
+        return ExpressionSelectMetaclass(
+            metaclass=metaclass, bases=bases, source_ref=source_ref
+        )
+
+
+def _makeRaiseExceptionMetaclassConflict():
+    return StatementRaiseException(
+        exception_type=ExpressionBuiltinMakeException(
+            exception_name="TypeError",
+            args=(
+                makeConstantRefNode(
+                    constant="""\
+metaclass conflict: the metaclass of a derived class must be a (non-strict) \
+subclass of the metaclasses of all its bases""",
+                    source_ref=internal_source_ref,
+                ),
+            ),
+            source_ref=internal_source_ref,
         ),
+        exception_value=None,
+        exception_trace=None,
+        exception_cause=None,
+        source_ref=internal_source_ref,
+    )
+
+
+# Note: This emulates selection of meta class based on base classes
+@once_decorator
+def getClassSelectMetaClassHelper():
+    helper_name = "_select_metaclass"
+
+    result = makeInternalHelperFunctionBody(
+        name=helper_name,
+        parameters=ParameterSpec(
+            ps_name=helper_name,
+            ps_normal_args=(
+                "metaclass",
+                "bases",
+            ),
+            ps_pos_only_args=(),
+            ps_list_star_arg=None,
+            ps_dict_star_arg=None,
+            ps_default_count=0,
+            ps_kw_only_args=(),
+        ),
+        inline_const_args=False,  # TODO: Allow this.
+    )
+
+    metaclass_variable = result.getVariableForAssignment(variable_name="metaclass")
+    bases_variable = result.getVariableForAssignment(variable_name="bases")
+
+    temp_scope = None
+
+    tmp_winner_variable = result.allocateTempVariable(temp_scope, "winner")
+    tmp_iter_variable = result.allocateTempVariable(temp_scope, "iter")
+    tmp_item_variable = result.allocateTempVariable(temp_scope, "base")
+    tmp_item_type_variable = result.allocateTempVariable(temp_scope, "base_type")
+
+    # For non-types, the metaclass cannot be overruled by bases.
+    non_type_case = StatementReturn(
+        expression=ExpressionVariableRef(
+            variable=metaclass_variable, source_ref=internal_source_ref
+        ),
+        source_ref=internal_source_ref,
+    )
+
+    type_loop_body = makeStatementsSequenceFromStatements(
+        makeTryExceptSingleHandlerNode(
+            tried=makeStatementAssignmentVariable(
+                variable=tmp_item_variable,
+                source=ExpressionBuiltinNext1(
+                    value=ExpressionTempVariableRef(
+                        variable=tmp_iter_variable, source_ref=internal_source_ref
+                    ),
+                    source_ref=internal_source_ref,
+                ),
+                source_ref=internal_source_ref,
+            ),
+            exception_name="StopIteration",
+            handler_body=StatementLoopBreak(source_ref=internal_source_ref),
+            source_ref=internal_source_ref,
+        ),
+        makeStatementAssignmentVariable(
+            variable=tmp_item_type_variable,
+            source=ExpressionBuiltinType1(
+                value=ExpressionTempVariableRef(
+                    variable=tmp_item_variable, source_ref=internal_source_ref
+                ),
+                source_ref=internal_source_ref,
+            ),
+            source_ref=internal_source_ref,
+        ),
+        makeStatementConditional(
+            condition=ExpressionSubtypeCheck(
+                left=ExpressionTempVariableRef(
+                    variable=tmp_winner_variable, source_ref=internal_source_ref
+                ),
+                right=ExpressionTempVariableRef(
+                    variable=tmp_item_type_variable, source_ref=internal_source_ref
+                ),
+                source_ref=internal_source_ref,
+            ),
+            yes_branch=None,  # Ignore if current winner is already a subtype.
+            no_branch=makeStatementConditional(
+                condition=ExpressionSubtypeCheck(
+                    left=ExpressionTempVariableRef(
+                        variable=tmp_item_type_variable, source_ref=internal_source_ref
+                    ),
+                    right=ExpressionTempVariableRef(
+                        variable=tmp_winner_variable, source_ref=internal_source_ref
+                    ),
+                    source_ref=internal_source_ref,
+                ),
+                yes_branch=makeStatementAssignmentVariable(
+                    variable=tmp_winner_variable,
+                    source=ExpressionTempVariableRef(
+                        variable=tmp_item_type_variable, source_ref=internal_source_ref
+                    ),
+                    source_ref=internal_source_ref,
+                ),
+                no_branch=_makeRaiseExceptionMetaclassConflict(),
+                source_ref=internal_source_ref,
+            ),
+            source_ref=internal_source_ref,
+        ),
+    )
+
+    type_case = makeStatementsSequenceFromStatements(
+        makeStatementAssignmentVariable(
+            variable=tmp_winner_variable,
+            source=ExpressionVariableRef(
+                variable=metaclass_variable, source_ref=internal_source_ref
+            ),
+            source_ref=internal_source_ref,
+        ),
+        makeStatementAssignmentVariable(
+            variable=tmp_iter_variable,
+            source=ExpressionBuiltinIter1(
+                value=ExpressionVariableRef(
+                    variable=bases_variable, source_ref=internal_source_ref
+                ),
+                source_ref=internal_source_ref,
+            ),
+            source_ref=internal_source_ref,
+        ),
+        StatementLoop(loop_body=type_loop_body, source_ref=internal_source_ref),
+        StatementReturn(
+            expression=ExpressionTempVariableRef(
+                variable=tmp_winner_variable, source_ref=internal_source_ref
+            ),
+            source_ref=internal_source_ref,
+        ),
+    )
+
+    tried = makeStatementConditional(
+        condition=ExpressionTypeCheck(
+            cls=ExpressionVariableRef(
+                variable=metaclass_variable, source_ref=internal_source_ref
+            ),
+            source_ref=internal_source_ref,
+        ),
+        yes_branch=type_case,
+        no_branch=non_type_case,
+        source_ref=internal_source_ref,
+    )
+
+    result.setChildBody(
+        makeStatementsSequenceFromStatement(
+            makeTryFinallyStatement(
+                provider=result,
+                tried=tried,
+                final=makeStatementsReleaseVariables(
+                    variables=(
+                        tmp_winner_variable,
+                        tmp_iter_variable,
+                        tmp_item_variable,
+                        tmp_item_type_variable,
+                    ),
+                    source_ref=internal_source_ref,
+                ),
+                source_ref=internal_source_ref,
+            )
+        )
     )
 
     return result
