@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2023, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -23,44 +23,45 @@ become nodes that build dictionaries.
 For Python3.5, unpacking can happen while creating dictionaries, these are
 being re-formulated to an internal function.
 
-Consult the developer manual for information. TODO: Add ability to sync
-source code comments with developer manual sections.
+Consult the Developer Manual for information. TODO: Add ability to sync
+source code comments with Developer Manual sections.
 
 """
 
-from nuitka.nodes.AssignNodes import (
-    StatementAssignmentVariable,
-    StatementReleaseVariable,
-)
-from nuitka.nodes.AttributeNodes import ExpressionAttributeLookup
+from nuitka.nodes.AttributeNodes import makeExpressionAttributeLookup
 from nuitka.nodes.BuiltinIteratorNodes import ExpressionBuiltinIter1
 from nuitka.nodes.BuiltinNextNodes import ExpressionBuiltinNext1
 from nuitka.nodes.ConstantRefNodes import makeConstantRefNode
 from nuitka.nodes.ContainerMakingNodes import makeExpressionMakeTuple
 from nuitka.nodes.DictionaryNodes import (
-    ExpressionKeyValuePair,
     StatementDictOperationUpdate,
     makeExpressionMakeDict,
     makeExpressionMakeDictOrConstant,
-    makeExpressionPairs,
 )
 from nuitka.nodes.ExceptionNodes import (
     ExpressionBuiltinMakeException,
     StatementRaiseException,
 )
 from nuitka.nodes.FunctionNodes import (
-    ExpressionFunctionCall,
-    ExpressionFunctionCreation,
     ExpressionFunctionRef,
+    makeExpressionFunctionCall,
+    makeExpressionFunctionCreation,
+)
+from nuitka.nodes.KeyValuePairNodes import (
+    makeExpressionKeyValuePair,
+    makeExpressionKeyValuePairConstantKey,
+    makeExpressionPairs,
 )
 from nuitka.nodes.LoopNodes import StatementLoop, StatementLoopBreak
 from nuitka.nodes.OperatorNodes import makeBinaryOperationNode
 from nuitka.nodes.ReturnNodes import StatementReturn
 from nuitka.nodes.TypeNodes import ExpressionBuiltinType1
+from nuitka.nodes.VariableAssignNodes import makeStatementAssignmentVariable
 from nuitka.nodes.VariableRefNodes import (
     ExpressionTempVariableRef,
     ExpressionVariableRef,
 )
+from nuitka.nodes.VariableReleaseNodes import makeStatementsReleaseVariables
 from nuitka.PythonVersions import python_version
 from nuitka.specs.ParameterSpecs import ParameterSpec
 
@@ -73,7 +74,7 @@ from .ReformulationTryExceptStatements import makeTryExceptSingleHandlerNode
 from .ReformulationTryFinallyStatements import makeTryFinallyStatement
 from .TreeHelpers import (
     buildNode,
-    buildNodeList,
+    buildNodeTuple,
     makeStatementsSequenceFromStatement,
     makeStatementsSequenceFromStatements,
 )
@@ -89,8 +90,8 @@ def buildDictionaryNode(provider, node, source_ref):
 
     return makeExpressionMakeDictOrConstant(
         pairs=makeExpressionPairs(
-            keys=buildNodeList(provider, node.keys, source_ref),
-            values=buildNodeList(provider, node.values, source_ref),
+            keys=buildNodeTuple(provider, node.keys, source_ref),
+            values=buildNodeTuple(provider, node.values, source_ref),
         ),
         user_provided=True,
         source_ref=source_ref,
@@ -122,7 +123,7 @@ def getDictUnpackingHelper():
 
     loop_body = makeStatementsSequenceFromStatements(
         makeTryExceptSingleHandlerNode(
-            tried=StatementAssignmentVariable(
+            tried=makeStatementAssignmentVariable(
                 variable=tmp_item_variable,
                 source=ExpressionBuiltinNext1(
                     value=ExpressionTempVariableRef(
@@ -161,7 +162,7 @@ def getDictUnpackingHelper():
                             ),
                             right=makeExpressionMakeTuple(
                                 elements=(
-                                    ExpressionAttributeLookup(
+                                    makeExpressionAttributeLookup(
                                         expression=ExpressionBuiltinType1(
                                             value=ExpressionTempVariableRef(
                                                 variable=tmp_item_variable,
@@ -191,20 +192,8 @@ def getDictUnpackingHelper():
 
     args_variable = result.getVariableForAssignment(variable_name="args")
 
-    final = (
-        StatementReleaseVariable(
-            variable=tmp_result_variable, source_ref=internal_source_ref
-        ),
-        StatementReleaseVariable(
-            variable=tmp_iter_variable, source_ref=internal_source_ref
-        ),
-        StatementReleaseVariable(
-            variable=tmp_item_variable, source_ref=internal_source_ref
-        ),
-    )
-
     tried = makeStatementsSequenceFromStatements(
-        StatementAssignmentVariable(
+        makeStatementAssignmentVariable(
             variable=tmp_iter_variable,
             source=ExpressionBuiltinIter1(
                 value=ExpressionVariableRef(
@@ -214,7 +203,7 @@ def getDictUnpackingHelper():
             ),
             source_ref=internal_source_ref,
         ),
-        StatementAssignmentVariable(
+        makeStatementAssignmentVariable(
             variable=tmp_result_variable,
             source=makeConstantRefNode(constant={}, source_ref=internal_source_ref),
             source_ref=internal_source_ref,
@@ -228,16 +217,22 @@ def getDictUnpackingHelper():
         ),
     )
 
-    result.setChild(
-        "body",
+    result.setChildBody(
         makeStatementsSequenceFromStatement(
             makeTryFinallyStatement(
                 provider=result,
                 tried=tried,
-                final=final,
+                final=makeStatementsReleaseVariables(
+                    variables=(
+                        tmp_result_variable,
+                        tmp_iter_variable,
+                        tmp_item_variable,
+                    ),
+                    source_ref=internal_source_ref,
+                ),
                 source_ref=internal_source_ref,
             )
-        ),
+        )
     )
 
     return result
@@ -255,12 +250,9 @@ def buildDictionaryUnpackingArgs(provider, keys, values, source_ref):
             result.append(
                 makeExpressionMakeDict(
                     pairs=(
-                        ExpressionKeyValuePair(
-                            key=makeConstantRefNode(
-                                constant=key, source_ref=source_ref
-                            ),
+                        makeExpressionKeyValuePairConstantKey(
+                            key=key,
                             value=buildNode(provider, value, source_ref),
-                            source_ref=source_ref,
                         ),
                     ),
                     source_ref=source_ref,
@@ -270,17 +262,16 @@ def buildDictionaryUnpackingArgs(provider, keys, values, source_ref):
             result.append(
                 makeExpressionMakeDict(
                     pairs=(
-                        ExpressionKeyValuePair(
+                        makeExpressionKeyValuePair(
                             key=buildNode(provider, key, source_ref),
                             value=buildNode(provider, value, source_ref),
-                            source_ref=source_ref,
                         ),
                     ),
                     source_ref=source_ref,
                 )
             )
 
-    return result
+    return tuple(result)
 
 
 def buildDictionaryUnpacking(provider, node, source_ref):
@@ -288,8 +279,8 @@ def buildDictionaryUnpacking(provider, node, source_ref):
         provider, node.keys, node.values, source_ref
     )
 
-    result = ExpressionFunctionCall(
-        function=ExpressionFunctionCreation(
+    result = makeExpressionFunctionCall(
+        function=makeExpressionFunctionCreation(
             function_ref=ExpressionFunctionRef(
                 function_body=getDictUnpackingHelper(), source_ref=source_ref
             ),
