@@ -1,4 +1,4 @@
-#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2023, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -389,9 +389,14 @@ def _buildMatchOr(provider, pattern, make_against, source_ref):
             source_ref=source_ref,
         )
 
-        or_condition_list.append(
-            makeAndNode(values=or_conditions, source_ref=source_ref)
-        )
+        if or_conditions:
+            or_condition_list.append(
+                makeAndNode(values=or_conditions, source_ref=source_ref)
+            )
+        else:
+            or_condition_list.append(
+                makeConstantRefNode(constant=True, source_ref=source_ref)
+            )
 
         # TODO: Inconsistency somewhere, can return empty.
         or_assignments_list.append(or_assignments or None)
@@ -439,7 +444,7 @@ def _buildMatchOr(provider, pattern, make_against, source_ref):
             provider=provider, name="match_or", source_ref=source_ref
         )
 
-        outline_body.setChild("body", body)
+        outline_body.setChildBody(body)
 
         conditions = (outline_body,)
         assignments = None
