@@ -1,4 +1,4 @@
-#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2023, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -17,12 +17,11 @@
 #
 """ Reformulation of try/except statements.
 
-Consult the developer manual for information. TODO: Add ability to sync
-source code comments with developer manual sections.
+Consult the Developer Manual for information. TODO: Add ability to sync
+source code comments with Developer Manual sections.
 
 """
 
-from nuitka.nodes.AssignNodes import StatementAssignmentVariable
 from nuitka.nodes.BuiltinRefNodes import ExpressionBuiltinExceptionRef
 from nuitka.nodes.ComparisonNodes import (
     ExpressionComparisonExceptionMatch,
@@ -41,6 +40,7 @@ from nuitka.nodes.StatementNodes import (
     StatementsSequence,
 )
 from nuitka.nodes.TryNodes import StatementTry
+from nuitka.nodes.VariableAssignNodes import makeStatementAssignmentVariable
 from nuitka.nodes.VariableRefNodes import ExpressionTempVariableRef
 from nuitka.PythonVersions import python_version
 
@@ -76,7 +76,7 @@ def makeTryExceptNoRaise(provider, temp_scope, tried, handling, no_raise, source
 
     statements = mergeStatements(
         (
-            StatementAssignmentVariable(
+            makeStatementAssignmentVariable(
                 variable=tmp_handler_indicator_variable,
                 source=makeConstantRefNode(constant=False, source_ref=source_ref),
                 source_ref=no_raise.getSourceReference(),
@@ -89,7 +89,7 @@ def makeTryExceptNoRaise(provider, temp_scope, tried, handling, no_raise, source
     handling = StatementsSequence(statements=statements, source_ref=source_ref)
 
     return makeStatementsSequenceFromStatements(
-        StatementAssignmentVariable(
+        makeStatementAssignmentVariable(
             variable=tmp_handler_indicator_variable,
             source=makeConstantRefNode(constant=True, source_ref=source_ref),
             source_ref=source_ref,
@@ -348,8 +348,7 @@ def buildTryExceptionNode(provider, node, source_ref):
             )
     else:
         if python_version < 0x300:
-            exception_handling.setChild(
-                "statements",
+            exception_handling.setChildStatements(
                 (
                     StatementPreserveFrameException(
                         preserver_id=0,  # unused with Python2
