@@ -1,4 +1,4 @@
-#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2023, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -503,7 +503,7 @@ def buildFrameNode(provider, nodes, code_object, source_ref):
         result = StatementsFrameFunction(
             statements=statements, code_object=code_object, source_ref=source_ref
         )
-    elif provider.isExpressionClassBody():
+    elif provider.isExpressionClassBodyBase():
         result = StatementsFrameClass(
             statements=statements,
             code_object=code_object,
