@@ -1,4 +1,4 @@
-#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2023, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -35,10 +35,12 @@ from .Utils import getArchitecture, isWin32OrPosixWindows, isWin32Windows
 # Cache, so we avoid repeated command lookups.
 _executable_command_cache = {}
 
+# We emulate and use APIs of stdlib, spell-checker: ignore popenargs,creationflags,preexec_fn,setsid
+
 
 def _getExecutablePath(filename, search_path):
     # Append ".exe" suffix  on Windows if not already present.
-    if isWin32OrPosixWindows() and not filename.lower().endswith(".exe"):
+    if isWin32OrPosixWindows() and not filename.lower().endswith((".exe", ".cmd")):
         filename += ".exe"
 
     # Now check in each path element, much like the shell will.
@@ -48,16 +50,22 @@ def _getExecutablePath(filename, search_path):
         path_element = path_element.strip('"')
         path_element = os.path.expanduser(path_element)
 
+        candidate = None
+
         if os.path.isfile(path_element):
             if os.path.normcase(os.path.basename(path_element)) == os.path.normcase(
                 filename
             ):
-                return path_element
+                candidate = path_element
         else:
             full = os.path.join(path_element, filename)
 
             if os.path.exists(full):
-                return full
+                candidate = full
+
+        if candidate is not None:
+            if os.access(candidate, os.X_OK):
+                return candidate
 
 
 def getExecutablePath(filename, extra_dir=None):
@@ -81,16 +89,16 @@ def isExecutableCommand(command):
 
 
 class NuitkaCalledProcessError(subprocess.CalledProcessError):
-    def __init__(self, retcode, cmd, output, stderr):
+    def __init__(self, exit_code, cmd, output, stderr):
         # False alarm, pylint: disable=super-init-not-called
 
-        subprocess.CalledProcessError(self, retcode, cmd)
+        subprocess.CalledProcessError(self, exit_code, cmd)
 
         # Python2 doesn't have this otherwise, but needs it.
         self.stderr = stderr
         self.output = output
         self.cmd = cmd
-        self.returncode = retcode
+        self.returncode = exit_code
 
     def __str__(self):
         result = subprocess.CalledProcessError.__str__(self)
@@ -123,14 +131,14 @@ def check_output(*popenargs, **kwargs):
     process = subprocess.Popen(stdout=subprocess.PIPE, *popenargs, **kwargs)
 
     output, stderr = process.communicate()
-    retcode = process.poll()
+    exit_code = process.poll()
 
-    if retcode:
+    if exit_code:
         cmd = kwargs.get("args")
         if cmd is None:
             cmd = popenargs[0]
 
-        raise NuitkaCalledProcessError(retcode, cmd, output=output, stderr=stderr)
+        raise NuitkaCalledProcessError(exit_code, cmd, output=output, stderr=stderr)
 
     return output
 
@@ -306,6 +314,7 @@ def getNullOutput():
 
 
 def getNullInput():
+    # spell-checker: ignore NULLDEV
     try:
         return subprocess.NULLDEV
     except AttributeError:
@@ -344,10 +353,12 @@ def executeToolChecked(logger, command, absence_message, stderr_filter=None):
             result = new_result
 
     if result != 0:
-        logger.sysexit("Error, call to %r failed: %s -> %s." % (tool, command, stderr))
+        logger.sysexit(
+            "Error, call to '%s' failed: %s -> %s." % (tool, command, stderr)
+        )
     elif stderr:
         logger.sysexit(
-            "Error, call to %r gave warnings: %s -> %s." % (tool, command, stderr)
+            "Error, call to '%s' gave warnings: %s -> %s." % (tool, command, stderr)
         )
 
     return stdout
@@ -361,10 +372,18 @@ def createProcess(
     stderr=None,
     shell=False,
     external_cwd=False,
+    new_group=False,
 ):
     if not env:
         env = os.environ
 
+    kw_args = {}
+    if new_group:
+        if isWin32Windows():
+            kw_args["creationflags"] = subprocess.CREATE_NEW_PROCESS_GROUP
+        else:
+            kw_args["preexec_fn"] = os.setsid
+
     process = subprocess.Popen(
         command,
         # Note: Empty string should also be allowed for stdin, therefore check
@@ -373,17 +392,20 @@ def createProcess(
         stdout=subprocess.PIPE if stdout is None else stdout,
         stderr=subprocess.PIPE if stderr is None else stderr,
         shell=shell,
-        # On Windows, closing file descriptions is now working with capturing outputs.
+        # On Windows, closing file descriptions is not working with capturing outputs.
         close_fds=not isWin32Windows(),
         env=env,
         # For tools that want short paths to work.
         cwd=getExternalUsePath(os.getcwd()) if external_cwd else None,
+        **kw_args
     )
 
     return process
 
 
-def executeProcess(command, env=None, stdin=False, shell=False, external_cwd=False):
+def executeProcess(
+    command, env=None, stdin=False, shell=False, external_cwd=False, timeout=None
+):
     process = createProcess(
         command=command, env=env, stdin=stdin, shell=shell, external_cwd=external_cwd
     )
@@ -395,6 +417,12 @@ def executeProcess(command, env=None, stdin=False, shell=False, external_cwd=Fal
     else:
         process_input = None
 
+    kw_args = {}
+    if timeout is not None:
+        # Apply timeout if possible.
+        if "timeout" in subprocess.Popen.communicate.__code__.co_varnames:
+            kw_args["timeout"] = timeout
+
     stdout, stderr = process.communicate(input=process_input)
     exit_code = process.wait()
 
