@@ -1,4 +1,4 @@
-#     Copyright 2022, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2023, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -145,7 +145,6 @@ def getWindowsRelease():
 def isDebianBasedLinux():
     dist_name, base, _dist_version = getLinuxDistribution()
 
-    # False alarm, pylint: disable=superfluous-parens
     return (base or dist_name) in ("Debian", "Ubuntu")
 
 
@@ -157,7 +156,10 @@ def isFedoraBasedLinux():
 
 def isAndroidBasedLinux():
     # spell-checker: ignore googlesource
-    return "ANDROID_ARGUMENT" in os.environ or "android.googlesource.com" in sys.version
+    if not isLinux():
+        return False
+
+    return "ANDROID_ROOT" in os.environ or "android.googlesource.com" in sys.version
 
 
 def isWin32Windows():
@@ -285,15 +287,14 @@ def hasStandaloneSupportedOS():
 
 
 def getUserName():
-    """Return the user name.
+    """Return the user name."""
+    # spell-checker: ignore getpwuid,getuid,getlogin
+    if isWin32Windows():
+        return os.getlogin()
+    else:
+        import pwd  # pylint: disable=I0021,import-error
 
-    Notes: Currently doesn't work on Windows.
-    """
-    import pwd  # pylint: disable=I0021,import-error
-
-    # spell-checker: ignore getpwuid,getuid
-
-    return pwd.getpwuid(os.getuid())[0]
+        return pwd.getpwuid(os.getuid())[0]
 
 
 @contextmanager
